/*
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *    "This product includes software developed by Dmitry Skavish 
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote 
 *    products derived from this software without prior written permission. 
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.api.sound;

import com.iv.flash.parser.*;
import com.iv.flash.util.*;
import java.io.PrintStream;
import com.iv.flash.api.*;

public class LazySound extends Sound {

    protected DataMarker data;

    public LazySound()	{}

    public static LazySound parse( Parser p ) {
        LazySound sound = new LazySound();
        sound.setID( p.getUWord() );
        p.initBits();       // 02/08/01 by sd
        sound.compressFormat = p.getBits(4);
        sound.rate = p.getBits(2);
        sound.isSample16bit = p.getBool();
        sound.isStereo = p.getBool();
        sound.sampleCount = p.getUDWord();
        // skip the data
        sound.data = new DataMarker( p.getBuf(), p.getPos(), p.getTagEndPos() );
        return sound;
    }

    public void write( FlashOutput fob ) {
        fob.writeTag( getTag(), 2+1+4+data.length() );
        fob.writeDefID( this );
        fob.initBits();
        fob.writeBits( compressFormat, 4 );
        fob.writeBits( rate, 2 );
        fob.writeBool( isSample16bit );
        fob.writeBool( isStereo );
        fob.flushBits();
        fob.writeDWord( sampleCount );
        data.write(fob);
    }

    public int getSize() {
        return data.buffer.length;
    }
    
    protected FlashItem copyInto( FlashItem item, ScriptCopier copier ) {
    
        super.copyInto( item, copier );
        
        ((LazySound)item).data = (DataMarker) data.getCopy();
        
        return item;
    }

    public FlashItem getCopy( ScriptCopier copier ) {
        return copyInto( new LazySound(), copier );
    }
}
