/*
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *    "This product includes software developed by Dmitry Skavish
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.commands;

import com.iv.flash.parser.*;
import com.iv.flash.api.*;
import com.iv.flash.util.*;
import com.iv.flash.log.*;

import java.io.*;
import java.util.*;

import org.apache.xpath.*;
import org.apache.xml.utils.*;
import org.w3c.dom.*;
import org.w3c.dom.traversal.*;
import org.xml.sax.SAXException;

public class GeneralXMLListCommand extends GenericXMLCommand {

    public static final int STARTX = 0;     // 20
    public static final int STARTY = 0;     // 20

    protected boolean mask;
    protected String  instancename;
    protected boolean isVertical;
    protected boolean isFixed;
    protected int order;    // 0 - none, -1 descending, +1 - ascending

    protected int listSize; // in twips
    protected int winWidth;
    protected int winHeight;

    protected XPath halign;
    protected XPath valign;
    protected XPath itemspace;
    protected XPath clip;
    protected XPath sortby;
    protected XPath itemname;

    protected void initParms( Context context ) throws IVException {
        super.initParms( context );
        mask         = getBoolParameter( context, "mask", true );
        instancename = getParameter( context, "instancename" );

        String orient = getParameter( context, "orient", "horizontal" );
        String spacing = getParameter( context, "spacing", "auto" );
        isVertical = orient.equalsIgnoreCase("vertical");
        isFixed    = spacing.equalsIgnoreCase("fixed");

        String sorder = getParameter( context, "order", "none" );
        if( sorder.equalsIgnoreCase("descending") ) order = -1;
        else if( sorder.equalsIgnoreCase("ascending") ) order = 1;
        else order = 0;

        halign       = getXPathParameter( context, "halign", "'left'" );
        valign       = getXPathParameter( context, "valign", "'top'" );
        itemspace    = getXPathParameter( context, "itemspace", "0" );
        clip         = getXPathParameter( context, "clip", "name()" );
        sortby       = getXPathParameter( context, "sortby", "'none'" );
        itemname     = getXPathParameter( context, "itemname", null );
    }

    protected Script makeList( FlashFile file, Context context, Script parent, int frameNum ) throws IVException {
        XMLContext xcontext = getXMLContext( file, context );

        NodeList nl = select( xcontext );

        Instance mainInst = getInstance();
        Matrix deRotate = deRotateMatrix( mainInst.matrix );
        Rect winBounds = deRotate.transform(new Rect(-1024, -1024, 1024, 1024)); // mask of the list
        winWidth = winBounds.getWidth();
        winHeight = winBounds.getHeight();

        Script listScript = new Script(1);
        Frame frame = listScript.newFrame();
        int x = STARTX, y = STARTY;

        // sort items
        ArrayList al = sort( nl, sortby, order == 1, order != 0 );
        // process datasource
        int layer = 1;
        int num = al.size();
        for( int i=0; i<num; i++ ) {
            Node node = ((SortListNode)al.get(i)).node;
            XMLContext myContext = new XMLContext(xcontext, node);

            String clipName = evalStringParameter( myContext, clip, "" );
            Script template = file.getScript(clipName);
            if( template == null ) {
                Log.log( "cmdScriptNotFound", clipName, getCommandName() );
            } else {
                Script myScript = template.copyScript();
                file.processScript( myScript, myContext );

                int delta = 0;
                if( itemspace != null ) {
                    delta = evalIntParameter( myContext, itemspace, 0 ) * 20;
                }
                Matrix matrix = new Matrix();
                matrix.translate(x,y);
                Instance scInst = frame.addInstance(myScript, layer++, matrix, null);
                if( itemname != null ) {
                    scInst.name = evalStringParameter( myContext, itemname, "item"+i );
                }

                Rect bounds = myScript.getBounds();
                int width = bounds.getWidth();
                int height = bounds.getHeight();
                if( isVertical ) {
                    if( !isFixed ) {
                        delta += height;
                    }
                    y += delta;
                } else {
                    if( !isFixed ) {
                        delta += width;
                    }
                    x += delta;
                }

                String s_halign = evalStringParameter( myContext, halign, "left" );
                String s_valign = evalStringParameter( myContext, valign, "top" );

                // alignment
                double shiftX, shiftY;
                if( isVertical ) {
                    shiftX = winWidth;
                    shiftY = height;
                } else {
                    shiftX = width;
                    shiftY = winHeight;
                }
                if( s_halign.equalsIgnoreCase("right") ) {
                    matrix.translateX( shiftX );
                } else if( s_halign.equalsIgnoreCase("center") ) {
                    matrix.translateX( shiftX/2 );
                }

                if( s_valign.equalsIgnoreCase("bottom") ) {
                    matrix.translateY( shiftY );
                } else if( s_valign.equalsIgnoreCase("center") ) {
                    matrix.translateY( shiftY/2 );
                }
            }
        }

        // calculate list size in twips
        if( isVertical ) {
            listSize = y-STARTY;
        } else {
            listSize = x-STARTX;
        }

        Matrix myMatrix = new Matrix();
        myMatrix.translate( -winWidth/2, -winHeight/2 );
        mainInst.matrix = deScaleMatrix( mainInst.matrix );
        mainInst.matrix = mainInst.matrix.multiply(myMatrix);

        if( instancename != null ) {
            mainInst.name = instancename;
        }

        return listScript;
    }

    protected void addMask( Script parent, int frameNum ) {
        if( mask ) {
            // adding mask
            Rect mask_rect = new Rect(0, 0, winWidth, winHeight);
            int maskDepth = parent.reserveLayers( getInstance().depth, 1 );
            Frame maskFrame = parent.getFrameAt(frameNum);
            Instance maskInst = maskFrame.addInstance( getMask(mask_rect), maskDepth, getInstance().matrix, null);
            maskInst.clip = getInstance().depth;
        }
    }
}
