package com.iv.flash.commands;

import com.iv.flash.parser.*;
import com.iv.flash.api.*;
import com.iv.flash.api.action.*;
import com.iv.flash.api.sound.*;
import com.iv.flash.api.shape.*;
import com.iv.flash.util.*;
import com.iv.flash.cache.*;
import com.iv.flash.url.*;
import com.iv.flash.log.*;
import java.io.*;

/**
 * Class to support the Insert MP3 generator template.<BR>
 *
 * @author James Taylor
 * @author Andrew Wason
 */

public class MP3SoundCommand extends GenericCommand
{
    public MP3SoundCommand() { }

    public void doCommand( FlashFile file, Context context, Script parent, int frame ) throws IVException
    {
        // Get object parameters

        String filename = getParameter( context, "filename", "" );
        boolean cache = getBoolParameter( context, "cache", false );
        boolean stream = getBoolParameter( context, "stream", false );
        boolean stopAction = getBoolParameter( context, "stopaction", true );
        int delay = getIntParameter( context, "delay", 0 );
        String instancename = getParameter( context, "instancename" );

        IVUrl url = IVUrl.newUrl( filename, file );

        Script script = getInstance().copyScript();

        Frame stopFrame = null;

        if ( stream )
        {
            SoundStreamBuilder ssb = null;

            try
            {
                if ( cache ) ssb = ( SoundStreamBuilder ) MediaCache.getMedia( url );
                
                if ( ssb == null ) 
                {
                    ssb = SoundStreamBuilder.newSoundStreamBuilder( url, file.getFrameRate() );
                }
                
                if ( cache ) MediaCache.addMedia( url, ssb, ssb.getSize(), cache );

                SoundStreamHead head = ssb.getSoundStreamHead();

                // Add the SoundStreamHead to the current frame in the parent script

                parent.getFrameAt( frame ).addFlashObject( head );

                int frameCount = parent.getFrameCount();
                
                int f = frame;

                SoundStreamBlock block;

                while ( ( block = ssb.getNextSoundStreamBlock() ) != null )
                {
                    if ( f >= frameCount )
                    {
                        parent.newFrame().addFlashObject( block );
                    }
                    else
                    {
                        parent.getFrameAt( f ).addFlashObject( block );
                    }

                    f++;
                }

                getInstance().def = Shape.getEmptyShape();
                
                stopFrame = parent.getFrameAt( f - 1 );
            }
            catch( IOException e )
            {
                throw new IVException( e, "errCmdFileRead", filename, getCommandName() );
            }
        }
        else
        {
            MP3Sound sound = null;
            
            try 
            {
                if ( cache ) sound = ( MP3Sound ) MediaCache.getMedia( url );
                
                if ( sound == null ) 
                {
                    sound = MP3Sound.newMP3Sound( url );
                }
                
                if ( cache ) MediaCache.addMedia( url, sound, sound.getSize(), cache );
                
            } 
            catch( IOException e ) 
            {
                throw new IVException( e, "errCmdFileRead", filename, getCommandName() );
            }

            // Set the delay if provided

            if ( delay != 0 )
            {
                sound.setDelaySeek( delay );
            }

            SoundInfo soundInfo = SoundInfo.newSoundInfo( 0 );

            StartSound startSound = StartSound.newStartSound( sound, soundInfo );

            Frame newFrame = script.newFrame();
            newFrame.addFlashObject( startSound );
            
            stopFrame = newFrame;
        }

        if( instancename != null )
        {
            getInstance().name = instancename;
        }

        if( stopAction )
        {
            DoAction action = new DoAction();
            
            action.program = new Program();
            action.program.stop();
            action.program.none();
            
            stopFrame.addFlashObject(action);
        }
    }
}
