/*
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *    "This product includes software developed by Dmitry Skavish
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.commands;

import com.iv.flash.parser.*;
import com.iv.flash.api.*;
import com.iv.flash.util.*;
import com.iv.flash.log.*;
import java.io.*;
import org.w3c.dom.*;

/**
 * Set environment in the following format:<BR>
 * name1,   name2,   name3<BR>
 * value11, value12, value13<BR>
 * value21, value22, value23<BR>
 * <P>Only first line of values is considered
 */
public class SetEnvironment2Command extends GenericCommand {

    public SetEnvironment2Command() {}

    public void doCommand( FlashFile file, Context context, Script parent, int frame ) throws IVException {
        String datasource = getParameter( context, "datasource" );
        if( datasource != null && datasource.length() > 0 ) {
            Object data;

/*            if( datasource.charAt(0) == '=' ) {
                String xpath = datasource.substring(1);
                XPath xpath = XMLHelper.getXPath();
            }*/

            // read datasource into either String[][] or xml Node
            try {
                data = DataSourceHelper.getContextData(datasource, file);
            } catch( IOException e ) {
                throw new IVException( e, "errDataRead", datasource, getCommandName() );
            }

            // create datasource
            // data is either Node or String[][]
            // we don't want to check for Node because if user does not use xml stuff it will be loaded and may cause
            // some problems, so we check for String[][]
            if( data instanceof String[][] ) {
                String[][] sdata = (String[][]) data;
                if( sdata.length < 1 ) {
                    throw new IVException( "invalDataSource", datasource, getCommandName() );
                }

                Context myContext;
                if( context instanceof FakeContext ) {
                    myContext = new StandardContext(context.getParent());
                    ((FakeContext)context).setContext( myContext );
                } else {
                    myContext = context;
                }

                if( sdata.length > 1 ) {
                    try {
                        for( int i=0; i<sdata[0].length; i++ ) {
                            myContext.setValue( context.apply(sdata[0][i]), context.apply(sdata[1][i]) );
                        }
                    } catch( ArrayIndexOutOfBoundsException e ) {
                        throw new IVException( "invalDataSource", datasource, getCommandName() );
                    }
                }
            } else {
                if( context instanceof FakeContext ) {
                    Context myContext = new XMLContext( context.getParent(), (Node) data );
                    ((FakeContext)context).setContext( myContext );
                }
            }
        }
    }

}
