/*
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *    "This product includes software developed by Dmitry Skavish 
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote 
 *    products derived from this software without prior written permission. 
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.commands;

import com.iv.flash.parser.*;
import com.iv.flash.api.*;
import com.iv.flash.util.*;

public class SetScaleCommand extends GenericCommand {

    public SetScaleCommand() {}

    private double parseNumber( String num, double def ) {
        if( num == null ) return def;
        try {
            return Double.valueOf(num).doubleValue();
        } catch( NumberFormatException e ) {
            return def;
        }
    }

    public void doCommand( FlashFile file, Context context, Script parent, int frame ) throws IVException {
        Instance inst = getInstance();
        Script script = inst.getScript();
        file.processScript( script, context );

        double xscale  = parseNumber( getParameter( context, "xscale" ),  1.0 );
        double yscale  = parseNumber( getParameter( context, "yscale" ),  1.0 );
        double xoffset = parseNumber( getParameter( context, "xoffset" ), 0.0 );
        double yoffset = parseNumber( getParameter( context, "yoffset" ), 0.0 );
        double rotate  = parseNumber( getParameter( context, "rotate" ),  0.0 );

        Matrix m = new Matrix();
        m.setScaleX(xscale);
        m.setScaleY(yscale);

        if( rotate != 0.0 ) {
            Matrix mm = new Matrix();
            double angle = (rotate/180)*Math.PI;
            double sin = Math.sin(angle);
            double cos = Math.cos(angle);
            mm.setScaleX(cos);
            mm.setScaleY(cos);
            mm.setRotateSkew0(sin);
            mm.setRotateSkew1(-sin);
            m = mm.multiply(m);
        }

        inst.matrix = inst.matrix.multiply(m);
        inst.matrix.setTranslateX( inst.matrix.getTranslateX()+xoffset*20.0 );
        inst.matrix.setTranslateY( inst.matrix.getTranslateY()+yoffset*20.0 );
    }

}
