/*
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *    "This product includes software developed by Dmitry Skavish
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.commands;

import com.iv.flash.parser.*;
import com.iv.flash.api.*;
import com.iv.flash.util.*;
import java.io.*;
import java.util.*;

import org.apache.xpath.*;
import org.apache.xml.utils.*;
import org.w3c.dom.*;
import org.w3c.dom.traversal.*;
import org.xml.sax.SAXException;

public class XMLReplicateMovieClipCommand extends GenericXMLCommand {

    public XMLReplicateMovieClipCommand() {}

    public void doCommand( FlashFile file, Context context, Script parent, int frame ) throws IVException {
        super.initParms( context );

        boolean expand = getBoolParameter( context, "expand", true );

        int order = 0;
        String sorder = getParameter( context, "order", "none" );
        if( sorder.equalsIgnoreCase("descending") ) order = -1;
        else if( sorder.equalsIgnoreCase("ascending") ) order = 1;
        else order = 0;

        XPath sortby   = getXPathParameter( context, "sortby", "'none'" );

        // read data and create context
        XMLContext xcontext = getXMLContext( file, context );

        // select nodes
        NodeList nl = select( xcontext );

        // sort items
        ArrayList al = sort( nl, sortby, order == 1, order != 0 );

        // get the script to be replicated
        Script template = getInstance().getScript();
        Script newScript = new Script(20);

        // process datasource
        int num = al.size();
        for( int i=0; i<num; i++ ) {
            Node node = ((SortListNode)al.get(i)).node;
            XMLContext myContext = new XMLContext(xcontext, node);

            Script myScript = template.copyScript();
            file.processScript( myScript, myContext );
            myScript.reserveLayers(i+1, newScript.getMaxDepth()); // shift all the layers up, this fix some flash player bug
            newScript.appendScript( myScript );
        }

        getInstance().setScript( newScript );
    }

}
