/*
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *    "This product includes software developed by Dmitry Skavish 
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote 
 *    products derived from this software without prior written permission. 
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.log;

import com.iv.flash.api.*;
import com.iv.flash.util.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.text.*;

public class FileLogger extends Logger {

    private String fileName;
    private IVVector entries = new IVVector();
    private boolean commit;
    private LogDaemon daemon;

    class LogDaemon extends Thread {
        public LogDaemon() {}

        public void run() {
            for(;;) {
                IVVector entries;
                while( (entries=takeEntries()) == null ) {
                    try {
                        Thread.sleep(1000); // sleep for 1 sec
                    } catch( InterruptedException e ) {
                        continue;
                    }
                }
                RandomAccessFile file = open();
                for( int i=0; i<entries.size(); i++ ) {
                    LogEntry entry = (LogEntry) entries.elementAt(i);
                    writeToLog( file, entry.getMessage(), entry.getDate() );
                }
                close( file );
            }
        }
    }

    public FileLogger( String fileName, boolean commit ) {
        File file = Util.getSysFile( fileName );
        this.fileName = file.getAbsolutePath();
        this.commit = commit;
        if( !commit ) {
            daemon = new LogDaemon();
            daemon.start();
        }
    }

    protected synchronized IVVector takeEntries() {
        if( entries.size() == 0 ) return null;
        IVVector v = entries;
        entries = new IVVector();
        return v;
    }

    public synchronized void log( String msg ) {
        if( commit ) writeToLog( msg, new Date() );
        else {
            entries.addElement( new LogEntry(msg) );
            //daemon.interrupt();
        }
    }

    protected void writeToLog( String msg, Date date ) {
        RandomAccessFile file = open();
        writeToLog( file, msg, date );
        close( file );
    }

    protected void writeToLog( RandomAccessFile file, String msg, Date date ) {
        if( msg == null || msg.length() == 0 ) return;
        SimpleDateFormat formatter = new SimpleDateFormat ("MM/dd/yyyy hh:mm:ss z");
        String dateStr = formatter.format( date );
        StringTokenizer st = new StringTokenizer(msg, "\r\n", false);
        while( st.hasMoreTokens() ) {
            String m = st.nextToken();
            m = dateStr + " : " + m + Util.lineSeparator;
            writeToLog( file, m );
        }
    }

    protected RandomAccessFile open() {
        try {
            RandomAccessFile raf = new RandomAccessFile( fileName, "rw" );
            raf.seek( raf.length());
            return raf;
        } catch( FileNotFoundException e ) {
            try {
                FileWriter writer = new FileWriter(fileName);
                writer.write( "<---------------------------  JGenerator Log file  ------------------------------->"+Util.lineSeparator );
                writer.close();
            } catch( IOException ee ) {
                Log.err( "Cannot create log file", ee );
                return null;
            }
            return open();
        }
        catch( IOException e ) {
            Log.err( e );
            return null;
        }
    }

    protected void close( RandomAccessFile file ) {
        try {
            file.close();
        } catch( IOException e ) {
            Log.err( e );
        }
    }

    protected void writeToLog( RandomAccessFile raf, String msg ) {
        try {
            raf.write( msg.getBytes() );
        } catch( IOException e ) {
            Log.err( msg, e );
        }
    }
}
