/*
 * $Id: JDBCUrl.java,v 1.5 2001/08/01 23:08:16 skavish Exp $
 *
 * ==========================================================================
 *
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *    "This product includes software developed by Dmitry Skavish
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.url;

import java.io.*;
import java.util.*;
import java.net.*;
import java.sql.*;
import com.iv.flash.util.*;
import com.iv.flash.api.*;
import com.iv.flash.log.*;

/**
 * Implementation of fgjdbc:// url
 * <P>
 * Syntax is: fgjdbc:///?driver=org.gjt.mm.mysql.Driver&url=jdbc:mysql://domen:3306/dbName&
 *            userid=username&password=password&query=select columnName from Table
 * <p>
 * The other parameters are (not supported now):
 * <UL>
 * <LI>lock
 * <LI>cache
 * <LI>share
 * <LI>timeout
 * </UL>
 *
 * @author Dmitry Skavish
 */
public class JDBCUrl extends IVUrl {

    private Hashtable parms;
    private String driver;
    private String url;
    private String userid;
    private String password;
    private String query;
    /*private String lock;
      private String cache;
      private String share;
      private String timeout;
    */

    public JDBCUrl( String surl ) throws IVException {
        parms = parseQueryString( surl );
        if( parms == null ) {
            throw new IVException( "invalUrl", surl );
        }
        driver   = (String) parms.get("driver");
        url      = (String) parms.get("url");
        userid   = (String) parms.get("userid");
        password = (String) parms.get("password");
        query    = (String) parms.get("query");
        /*  lock     = (String) parms.get("lock");
            cache    = (String) parms.get("cache");
            share    = (String) parms.get("share");
            timeout  = (String) parms.get("timeout");
        */
        if( driver == null || url == null || query == null ) {
            throw new IVException( "invalUrl", surl );
        }
    }

    public String getParameter( String name ) {
        return (String) parms.get( name );
    }

    public String getName() {
        return url+"@"+userid+"/"+query;
    }

    public InputStream getInputStream() throws IOException {
        String[][] data = getData();
        StringBuffer sb = new StringBuffer( data.length*data[0].length*10 );
        for( int i=0; i<data.length; i++ ) {
            int ilen = data[i].length;
            for( int j=0; j<ilen; j++ ) {
                String s = data[i][j];
                if( s == null ) sb.append("\"\"");
                else {
                    sb.append("\"");
                    sb.append(s);
                    sb.append("\"");
                }
                if( j+1 < ilen ) sb.append(',');
            }
            sb.append('\n');
        }
        String str = new String( sb );
        return new ByteArrayInputStream( str.getBytes() );
    }

    public boolean hasDataReady() {
        return true;
    }

    public String[][] getData() throws IOException {
        try {
            Class.forName(driver);
        } catch( Exception e ) {
            throw new IOException("driver not found");
        }

        String[][] data = null;

        try {
            Connection con;
            if( userid == null ) {
                con = DriverManager.getConnection(url);
            } else {
                con = DriverManager.getConnection(url, userid, password);
            }
            Statement stmt = con.createStatement();

            ResultSet rs = stmt.executeQuery(query);
            ResultSetMetaData meta = rs.getMetaData();

            int numCols = meta.getColumnCount();
            String[] header = new String[numCols];
            for( int i=0; i<numCols; i++ ) {
                header[i] = meta.getColumnLabel(i+1);
            }

            IVVector lines = new IVVector();
            while( rs.next() ) {
                String[] line = new String[numCols];
                for( int i=0; i<numCols; i++ ) {
                    Object o = rs.getObject(i+1);
                    String s = res2string(o);
                    line[i] = Util.processEscapes(s);
                }
                lines.addElement(line);
            }

            data = new String[lines.size()+1][];
            data[0] = header;
            for( int i=0; i<lines.size(); i++ ) {
                data[i+1] = (String[]) lines.elementAt(i);
            }

            con.close();
            return data;
        } catch( Exception e ) {
            throw new IOException( e.getMessage() );
        }
    }

    private String res2string( Object o ) {
        if( o == null ) return "";
        if( o instanceof byte[] ) return new String((byte[])o);
        return o.toString();
    }
}
