/*
 * $Id: CommandExecutor.java,v 1.4 2001/08/13 22:45:43 skavish Exp $
 *
 * ===========================================================================
 *
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *    "This product includes software developed by Dmitry Skavish
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.util;

import com.iv.flash.api.*;
import java.util.*;
import java.text.*;
import java.lang.reflect.*;


/**
 * Standard executor of context commands.
 * <P>
 * To add new command you have to add new method to this class
 * or to one of its subclasses. Name of the command will be name of the method.
 * If command was invoked without parameters then method without parameters will be
 * called, if command was invoked with several parameters then method with corresponding
 * number of parameters will be called. If none of these was succeded then method with
 * parameter Vector will be called.
 *
 * @author Dmitry Skavish
 * @see CommandContext
 */
public class CommandExecutor {

    protected FlashFile file;

    /**
     * Create executor in environment of given flash file.
     *
     * @param file   environment for executor
     */
    public CommandExecutor( FlashFile file ) {
        this.file = file;
    }

    public CommandExecutor() {
    }

    public void setFlashFile( FlashFile file ) {
        this.file = file;
    }

    public FlashFile getFlashFile() {
        return file;
    }

    /**
     * Executes command.
     *
     * @param name   name fo the command
     * @param parms  parameters or null
     * @return result of the command execution
     */
    public String execute( String name, Vector parms ) {
        try {
            Class c = getClass();
            Method m;
            Object[] mp;
            try {
                if( parms == null ) {
                    m = c.getMethod( name, null );
                    mp = null;
                } else {
                    Class[] cmp = new Class[parms.size()];
                    for( int i=0; i<cmp.length; i++ )
                        cmp[i] = String.class;
                    m = c.getMethod( name, cmp );
                    mp = new String[cmp.length];
                    for( int i=0; i<cmp.length; i++ )
                        mp[i] = (String) parms.elementAt(i);
                }
            } catch( NoSuchMethodException e ) {
                m = c.getMethod( name, new Class[] {Vector.class} );
                mp = new Object[] {parms};
            }
            Object res = m.invoke( this, mp );
            if( res == null ) return "";
            return res.toString();
        } catch( Exception e ) {
            return "";
        }
    }

    /*---------------------------------------------------------------------------------*/
    /*                                   C O M M A N D S                               */
    /*---------------------------------------------------------------------------------*/

    /**
     * Returns current time and date
     *
     * @return current time and date
     */
    public String date() {
        return new Date().toLocaleString();
    }

    /**
     * Returns current time and date formatted according to given template.<p>
     * Examples:
     * <UL>
     * <LI>MM/dd/yyyy hh:mm:ss z
     * </UL>
     *
     * @param format date format
     * @return current formatted date
     */
    public String date( String format ) {
        SimpleDateFormat formatter = new SimpleDateFormat(format);
        return formatter.format( new Date() );
    }

    /**
     * Return JGenerator version
     *
     * @return JGenerator version
     */
    public String version() {
        return Util.getVersion();
    }

}



