/*
 * $Id: IVException.java,v 1.6 2001/08/02 22:26:33 skavish Exp $
 *
 * ===========================================================================
 *
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *    "This product includes software developed by Dmitry Skavish
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.util;

import com.iv.flash.log.*;

/**
 * This exception is thrown if there any errors during generation process.
 * <P>
 * The exception can be created with message from resource file: bin/message.txt.
 * All messages in this file are defined by their keys which are to be used when throwing the exception.
 * <p>
 * Usage:<BR>
 * <UL>
 * <LI>wrap some other exception into IVException and rethrow it with a message from resorce and parameters:<br>
 *     <pre><CODE>
 *     ...
 *     } catch( IOException e ) {
 *       throw new IVException( e, "errCmdFileRead", url.getName(), getCommandName() );
 *     }
 *     </CODE></pre>
 * <LI><CODE>throw new IVException( "infiniteLoop" );</CODE>
 * </UL>
 *
 * @author Dmitry Skavish
 */
public class IVException extends Exception {

    private String key;
    private Exception nestedException;
    private Object[] parms;

    public IVException( Exception e, String key ) {
        this.nestedException = e;
        this.key = key;
    }

    public IVException( String key ) {
        this( (Exception) null, key );
    }

    public IVException( Exception e ) {
        this( e, (String) null );
    }

    public IVException( Exception e, String key, Object p0 ) {
        this( e, key );
        parms = new Object[] { p0 };
    }

    public IVException( Exception e, String key, Object p0, Object p1 ) {
        this( e, key );
        parms = new Object[] { p0, p1 };
    }

    public IVException( Exception e, String key, Object p0, Object p1, Object p2 ) {
        this( e, key );
        parms = new Object[] { p0, p1, p2 };
    }

    public IVException( Exception e, String key, Object[] p ) {
        this( e, key );
        parms = p;
    }

    public IVException( String key, Object p0 ) {
        this( (Exception) null, key );
        parms = new Object[] { p0 };
    }

    public IVException( String key, Object p0, Object p1 ) {
        this( (Exception) null, key );
        parms = new Object[] { p0, p1 };
    }

    public IVException( String key, Object p0, Object p1, Object p2 ) {
        this( (Exception) null, key );
        parms = new Object[] { p0, p1, p2 };
    }

    public IVException( String key, Object[] p ) {
        this( (Exception) null, key );
        parms = p;
    }

    public IVException() {}

    public Exception getNestedException() {
        return this.nestedException;
    }

    public Object[] getParameters() {
        return this.parms;
    }

    public String getMessageKey() {
        return key;
    }

    public String getMessage() {
        String msg = Log.getMessage( key, parms );
        if( msg == null || msg.length() == 0 ) return "";
        return msg;
    }
}
