/*
 * $Id: StandardContext.java,v 1.7 2001/08/02 22:26:34 skavish Exp $
 *
 * ===========================================================================
 *
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *    "This product includes software developed by Dmitry Skavish
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.util;

import com.iv.flash.api.*;
import java.util.*;

/**
 * Standard generator context of name,value pairs.
 *
 * @author Dmitry Skavish
 * @see XMLContext
 */
public class StandardContext extends Context {

    private Hashtable table = new Hashtable();

    /**
     * Creates empty context.
     */
    public StandardContext() {}

    /**
     * Creates context from specified hashtable.
     *
     * @param table  hashtable to be used as storage for name,value pairs
     */
    public StandardContext( Hashtable table ) {
        this.table = table;
    }

    /**
     * Creates context in specifed parent context.
     *
     * @param context parent context
     */
    public StandardContext( Context context ) {
        setParent( context );
    }

    /**
     * Puts specified value under specified name into this context.
     *
     * @param name   the specified name
     * @param value  the specified value
     */
    public void setValue( String name, String value ) {
        if( !PropertyManager.varCaseSensitive ) name = name.toUpperCase();
        table.put(name, value);
    }

    /**
     * Returns value by name from this context or from it's parent context.
     *
     * @param name   name of the value to be retrieved
     * @return value of the specified name
     */
    public String _getValue( String name ) {
        String value = (String) table.get(name);
        if( value != null ) return value;
        Context parent = getParent();
        if( parent != null ) return parent._getValue( name );
        return null;
    }

    protected StandardContext( StandardContext from ) {
        table = (Hashtable) from.table.clone();
        setParent( from.getParent() );
    }

    public Object clone() {
        return new StandardContext(this);
    }

}
