/*
 * $Id: XMLHelper.java,v 1.7 2001/08/02 22:26:34 skavish Exp $
 *
 * ===========================================================================
 *
 * The JGenerator Software License, Version 1.0
 *
 * Copyright (c) 2000 Dmitry Skavish (skavish@usa.net). All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *    "This product includes software developed by Dmitry Skavish
 *     (skavish@usa.net, http://www.flashgap.com/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The name "The JGenerator" must not be used to endorse or promote
 *    products derived from this software without prior written permission.
 *    For written permission, please contact skavish@usa.net.
 *
 * 5. Products derived from this software may not be called "The JGenerator"
 *    nor may "The JGenerator" appear in their names without prior written
 *    permission of Dmitry Skavish.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL DMITRY SKAVISH OR THE OTHER
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

package com.iv.flash.util;

import java.io.*;
import java.net.*;
import java.util.*;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Properties;

import org.apache.xerces.parsers.DOMParser;
import org.apache.xpath.*;
import org.apache.xpath.objects.*;
import org.w3c.dom.*;
import org.w3c.dom.traversal.*;
import org.xml.sax.SAXException;
import org.xml.sax.InputSource;

import javax.xml.parsers.*;
import javax.xml.transform.*;

import com.iv.flash.api.*;
import com.iv.flash.url.*;
import com.iv.flash.log.*;
import com.iv.flash.cache.*;

import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

/**
 * XMl Helper class.
 * <P>
 * Provides various methods for XML stuff
 *
 * @author Dmitry Skavish
 */
public class XMLHelper {

    private static DocumentBuilder dbuilder;
    private static XMLPrefixResolver defaultResolver = new XMLPrefixResolver();
    private static Hashtable xpaths = new Hashtable();

    static {
        DocumentBuilderFactory factory;
        try {
            System.setProperty( "javax.xml.parsers.DocumentBuilderFactory", DocumentBuilderFactoryImpl.class.getName() );
            factory = DocumentBuilderFactory.newInstance();
        } catch( java.security.AccessControlException e ) {
            factory = new DocumentBuilderFactoryImpl();
        }

        try {
            factory.setExpandEntityReferences(false);
            factory.setValidating(false);
            dbuilder = factory.newDocumentBuilder();
        } catch( javax.xml.parsers.ParserConfigurationException e ) {
            Log.log(e);
        }
    }

    public static DocumentBuilder getDocumentBuilder() {
        return dbuilder;
    }

    public static XMLPrefixResolver getPrefixResolver( Node node ) {
        return defaultResolver;
    }

    /**
     * Compiles specified string into XPath.
     * <P>
     * Tries to retrieve compiled XPath expression from cache,
     * if it's not in the cache then compiles give string into XPath
     * and caches in the cache.
     *
     * @param expr   string representing XPath expression
     * @return compiled XPath expression
     * @exception TransformerException
     */
    public synchronized static XPath getXPath( String expr ) throws TransformerException {
        XPath xpath = (XPath) xpaths.get( expr );
        if( xpath != null ) return xpath;
        xpath = new XPath(expr, null, defaultResolver, XPath.SELECT, null);
        xpaths.put( expr, xpath );
        return xpath;
    }

    /**
     * Evaluates specified XPath expression in the specified xml node.
     *
     * @param xpathContext
     * @param node   xml node to evaluated on
     * @param expr   xpath expression
     * @return result of xpath execution
     * @exception TransformerException
     */
    public static XObject evalXPath( XPathContext xpathContext, Node node, String expr )
        throws TransformerException
    {
        return evalXPath( xpathContext, node, getXPath( expr ) );
    }

    /**
     * Evaluates specified XPath expression in the specified xml node.
     *
     * @param xpathContext
     * @param node   xml node to evaluated on
     * @param xpath  xpath expression
     * @return result of xpath execution
     * @exception TransformerException
     */
    public static XObject evalXPath( XPathContext xpathContext, Node node, XPath xpath )
        throws TransformerException
    {
        return xpath.execute(xpathContext, node, getPrefixResolver(node));
    }

    /**
     * Evaluates XPath to list of nodes.
     *
     * @param xpathContext
     * @param node   xml node to evaluated on
     * @param expr   XPath expression
     * @return result of xpath execution - list of nodes
     * @exception TransformerException
     */
    public static NodeList selectNodeList( XPathContext xpathContext, Node node, String expr )
        throws TransformerException
    {
        return getNodeList( evalXPath(xpathContext, node, expr) );
    }

    /**
     * Evaluates XPath to list of nodes.
     *
     * @param xpathContext
     * @param node   xml node to evaluated on
     * @param xpath  XPath expression
     * @return result of xpath execution - list of nodes
     * @exception TransformerException
     */
    public static NodeList selectNodeList( XPathContext xpathContext, Node node, XPath xpath )
        throws TransformerException
    {
        return getNodeList( evalXPath(xpathContext, node, xpath) );
    }

    /**
     * Evaluates XPath to one node.
     *
     * @param xpathContext
     * @param node   xml node to be evaluated on
     * @param expr   XPath expression
     * @return xml node
     * @exception TransformerException
     */
    public static Node selectSingleNode( XPathContext xpathContext, Node node, String expr )
        throws TransformerException
    {
        NodeList nl = selectNodeList(xpathContext, node, expr);
        if( nl.getLength() > 0 ) return nl.item(0);
        return null;
    }

    /**
     * Evaluates XPath to one node.
     *
     * @param xpathContext
     * @param node   xml node to be evaluated on
     * @param xpath   XPath expression
     * @return xml node
     * @exception TransformerException
     */
    public static Node selectSingleNode( XPathContext xpathContext, Node node, XPath xpath )
        throws TransformerException
    {
        NodeList nl = selectNodeList(xpathContext, node, xpath);
        if( nl.getLength() > 0 ) return nl.item(0);
        return null;
    }

    /**
     * Converts result of XPath evaluation to nodelist.
     *
     * @param xo     result of xpath evaluation
     * @return nodelist
     * @exception TransformerException
     */
    public static NodeList getNodeList( XObject xo ) throws TransformerException {
        //NodeSet nodeset = xo.mutableNodeset();
        NodeSetDTM nodeset = xo.mutableNodeset();

//        if( !nodeset.getShouldCacheNodes() )
//            nodeset.setShouldCacheNodes(true);
        return (NodeList) nodeset;
    }

    /**
     * Returns String representation of specified xml node.
     *
     * @param node   the specified xml node
     * @return string representation of the node
     */
    public static String getNodeData( Node node ) {
        switch( node.getNodeType() ) {
            case Node.DOCUMENT_FRAGMENT_NODE:
            case Node.DOCUMENT_NODE:
            case Node.ELEMENT_NODE: {
              /*for (Node child = node.getFirstChild(); null != child;
                    child = child.getNextSibling())*/
                Node child = node.getFirstChild();
                if( child != null ) return getNodeData( child );
            }
            break;
            case Node.TEXT_NODE:
            case Node.CDATA_SECTION_NODE:
                return node.getNodeValue();
            case Node.ATTRIBUTE_NODE:
                return node.getNodeValue();
            case Node.PROCESSING_INSTRUCTION_NODE :
                break;
            default:
                break;
        }
        return "";
    }

    public static String getXObjectData( XObject xo ) {
        return xo.toString();
/*        switch( xo.getType() ) {
            case XObject.CLASS_BOOLEAN:
            case XObject.CLASS_STRING:
            case XObject.CLASS_NODESET:
            case XObject.CLASS_NULL:
            case XObject.CLASS_NUMBER:
            case XObject.CLASS_UNKNOWN:
            case XObject.CLASS_UNRESOLVEDVARIABLE:
            case XObject.CLASS_RTREEFRAG:
        }*/
    }

    /**
     * Parses input stream of some xml data into xml document.
     *
     * @param is     xml input stream
     * @return xml document
     * @exception IOException
     * @exception SAXException
     */
    public static Document parse( InputStream is ) throws IOException, SAXException {
        return parse( new InputSource(is) );
    }

    /**
     * Parses input source of some xml data into xml document.
     *
     * @param isrc   xml input source
     * @return xml document
     * @exception IOException
     * @exception SAXException
     */
    public static Document parse( InputSource isrc ) throws IOException, SAXException {
        return getDocumentBuilder().parse( isrc );
    }

    /**
     * Parses xml document specified by url.
     * <P>
     * First tries to retrieve xml document from cache.
     * If it is not cached then reads it, parses it and caches (if needed).
     *
     * @param url    url to xml document
     * @return xml document
     * @exception IOException
     * @exception SAXException
     */
    public static Document getDocument( IVUrl url ) throws IOException, SAXException {
        Document doc = XMLCache.getXMLDocument( url );
        if( doc != null ) return doc;
        doc = parse( url.getInputStream() );
        XMLCache.addXMLDocument( url, doc );
        return doc;
    }

    /**
     * Parses xml document specified by url and resolves it to one node.
     * <P>
     * First tries to retrieve xml document from cache.
     * If it is not cached then reads it, parses it and caches (if needed).
     * Then if there is no any xpath expression in url returns Document itself as a xml node.
     * If there is some Xpath then evaluates this XPath to one xml node and returns it.
     * <P>
     * The url's format: <url itself>#<xpath expression which returns single node><BR>
     * Example: <CODE>http://myserver.com/mydata.xml#/doc/item/firstNode</CODE>
     *
     * @param url    url to xml document
     * @return xml node
     * @exception IOException
     * @exception SAXException
     */
    public static Node getNode( IVUrl url ) throws IOException, SAXException {
        Document doc = getDocument( url );
        String ref = url.getRef();
        if( ref == null ) return doc;
        Node node;
        try {
            node = selectSingleNode( new XPathContext(), doc, ref );
        } catch( Exception e ) {
            throw new SAXException(e);
        }
        return node;
    }

}
