/*
** Copyright (C) 2001,2002 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
 *  $Revision: 2 $
 *  $Modtime: 11/22/02 11:53p $
 *  Soft Version : 2.1
 */
package faust.sacha.util;

import java.util.*;
import java.io.*;

public class FileModeList extends Thread implements List {

    private static final int STATE_IN_PROCESS   =   0;
    private static final int STATE_STOP         =   1;
    private static final int STATE_FINISHED     =   2;

    private static final String TMP_PREFIX      =   "ioArray";

    private Writer m_writer;
    private File m_file;
    //private ToWriteQueue m_toWriteQueue;
    private Object m_stateMutex;
    private int m_state;
    private int m_nbElements;
    private Object m_nbElementMutex;

    //private Thread m_thread;

    public FileModeList() throws IOException {
        //File tmpFile = null;

        //m_toWriteQueue = new ToWriteQueue();
        m_stateMutex = new Object();
        m_state = STATE_IN_PROCESS;
        m_nbElements = 0;
        m_nbElementMutex = new Object();

        // create a tmp file to write to
        m_file = File.createTempFile(TMP_PREFIX, null);
        //  make sure it's deleted once the JVM exit
        m_file.deleteOnExit();
        // create the writer
        m_writer = new BufferedWriter( new FileWriter(m_file) );
    }
//------------------------------------------------------------------------------
    public void run(){
        super.run();

        System.err.println("FileModeList::run() : started");

        //m_toWriteQueue.start();

        process();

    }
//------------------------------------------------------------------------------
    public void process(){
        Object obj;
        int state;

        while( true ){

            synchronized( m_stateMutex ){
                state = m_state;
            }

            if( state == STATE_IN_PROCESS ){
                try{
                    wait(500);
                }
                catch( InterruptedException intEx ){
                }
            }
            else
                break;

//            obj = m_toWriteQueue.get();
//
//            if( obj == null ){
//                System.err.println("process : got null");
//                continue;
//            }
//
//            write(obj);
        }
    }
//------------------------------------------------------------------------------
    public void setState( int state ){
        synchronized( m_stateMutex ){
            m_state = state;
        }

        notifyAll();

    }
//------------------------------------------------------------------------------
    private void write( Object obj ){

        if( contains(obj) )
            return;

        synchronized( m_writer ){
            try{
                m_writer.write( obj.toString() + "\n" );
                m_writer.flush();

                synchronized( m_nbElementMutex ){
                    m_nbElements++;
                }
            } catch(IOException e){
                System.err.println("FileModeList error writing object to file");
                e.printStackTrace();
            }
        }
    }
//------------------------------------------------------------------------------
    public boolean add(Object o) {
        //System.err.println("add : " + o.toString());
        //m_toWriteQueue.add( o );
//
        if( contains(o) ){
            return false;
        }

        write(o);

        // TOFIX : RETURN CRAP
        return true;
    }
//------------------------------------------------------------------------------
    public void add(int index, Object element) {

        //TOFIX : NOT IMPLEMENTED
    }
//------------------------------------------------------------------------------
    public boolean addAll(Collection c) {
        //TOFIX : NOT IMPLEMENTED

        return true;
    }
//------------------------------------------------------------------------------
    public boolean addAll(int index, Collection c) {

        //TOFIX : NOT IMPLEMENTED

        return true;
    }
//------------------------------------------------------------------------------
    public void clear() {
    }
//------------------------------------------------------------------------------
    public boolean contains(Object o) {

        BufferedReader searchFile = null;
        FileReader fileReader = null;
        String tmpStr = null;

//        if( m_toWriteQueue.contains(o) )
//            return true;

        synchronized( m_nbElementMutex ){
            if( m_nbElements == 0 )
                return false;
        }

        try{
            synchronized( m_file ){
                searchFile = new BufferedReader( new FileReader(m_file) );
                while( (tmpStr = searchFile.readLine()) != null ){
                    //if( tmpStr.equals( o.toString() ) ){
                    if( tmpStr.equalsIgnoreCase( o.toString() ) ){
                        try{
                            searchFile.close();
                        }
                        catch( IOException closeEx ){
                        }

                        return true;
                    }
                }
            }
        }
         catch( IOException ioEx ){
             if( searchFile != null ){
                try{
                    searchFile.close();
                }
                catch( IOException closeEx ){
                }
             }
            return false;
        }


        try{
            searchFile.close();
        }
        catch( IOException closeEx ){
        }

        return false;
    }
//------------------------------------------------------------------------------
    public boolean containsAll(Collection c) {

        return false;
    }
//------------------------------------------------------------------------------
    public Object get(int index) {
        BufferedReader searchFile = null;
        FileReader fileReader = null;
        String tmpStr = null;
        int arrayIndex;

        synchronized( m_nbElementMutex ){
            if( index > m_nbElements )
                return null;
            else{
                synchronized( m_file ){
                    try{
                        searchFile = new BufferedReader( new FileReader(m_file) );
                    }
                    catch( FileNotFoundException notFoundEx ){
                        return null;
                    }

                    for( arrayIndex = 0; arrayIndex <= index; arrayIndex++ ){
                        try{
                            tmpStr = searchFile.readLine();
                        }
                        catch( IOException ioEx ){
                            return null;
                        }
                    }
                }
            }
        }

        return (Object)tmpStr;
    }
//------------------------------------------------------------------------------
    public int indexOf(Object o) {

        //TOFIX NOT IMPLEMENTED
        return 0;
    }
//------------------------------------------------------------------------------
    public boolean isEmpty() {

        return false;
    }
//------------------------------------------------------------------------------
    public Iterator iterator() {

        //TOFIX : NOT IMPLEMENTED
        return null;
    }
//------------------------------------------------------------------------------
    public int lastIndexOf(Object o) {

        //TOFIX : NOT IMPLEMENTED
        return 0;
    }
//------------------------------------------------------------------------------
    public ListIterator listIterator() {

        //TOFIX : NOT IMPLEMENTED
        return null;
    }
//------------------------------------------------------------------------------
    public ListIterator listIterator(int index) {

        //TOFIX : NOT IMPLEMENTED
        return null;
    }
//------------------------------------------------------------------------------
    public Object remove(int index) {

        //TOFIX : NOT IMPLEMENTED
        return null;
    }
//------------------------------------------------------------------------------
    public boolean remove(Object o) {

        //TOFIX : NOT IMPLEMENTED
        return true;
    }
//------------------------------------------------------------------------------
    public boolean removeAll(Collection c) {

        //TOFIX : NOT IMPLEMENTED
        return true;
    }
//------------------------------------------------------------------------------
    public boolean retainAll(Collection c) {

        //TOFIX : NOT IMPLEMENTED
        return true;
    }
//------------------------------------------------------------------------------
    public Object set(int index, Object element) {

        //TOFIX : NOT IMPLEMENTED
        return null;
    }
//------------------------------------------------------------------------------
    public int size() {

        return m_nbElements;
    }
//------------------------------------------------------------------------------
    public List subList(int fromIndex, int toIndex) {

        //TOFIX : NOT IMPLEMENTED
        return null;
    }
//------------------------------------------------------------------------------
    public Object[] toArray() {

        //TOFIX : NOT IMPLEMENTED
        return null;
    }
//------------------------------------------------------------------------------
    public Object[] toArray(Object[] a) {

        //TOFIX : NOT IMPLEMENTED
        return null;
    }
//------------------------------------------------------------------------------
}



//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//class ToWriteQueue extends Thread {
//
//    public static final int STATE_NOT_STARTED   =   0;
//    public static final int STATE_STOP          =   1;
//    public static final int STATE_IN_PROCESS    =   2;
//
//    private QueueNode m_head;
//    private QueueNode m_tail;
//    private long m_size;
//    private Object m_queueMutex;
//    private int m_state;
//
//    public ToWriteQueue(){
//        m_head = null;
//        m_tail = null;
//        m_size = 0;
//        m_queueMutex = new Object();
//        m_state = STATE_NOT_STARTED;
//    }
////------------------------------------------------------------------------------
//    public void run(){
//        super.run();
//
//        System.err.println("ToWriteQueue::run() : started");
//
//        m_state = STATE_IN_PROCESS;
//
//        while( m_state != STATE_STOP ){
//            try{
//                wait(100);
//            }
//            catch( InterruptedException intEx ){
//            }
//        }
//    }
////------------------------------------------------------------------------------
//    public void add( Object object ){
//        QueueNode node = null;
//
//        //System.err.println("queue : add : " + object.toString());
//
//        if( object == null )
//            return;
//
//        if( contains(object) )
//            return;
//
//        node = new QueueNode(object);
//
//        synchronized( m_queueMutex ){
//            if( m_size == 0 ){
//                m_head = node;
//                m_tail = node;
//            }
//            else{
//                m_tail.next = node;
//                node.prev = m_tail;
//                m_tail = node;
//            }
//
//            m_size++;
//        }
//
//
//        /*
//         *  wake up anything that might be waiting for something to be added
//         */
//        //notifyAll();
//    }
////------------------------------------------------------------------------------
//    public Object get(){
//        Object ojb = null;
//        int nbRetries = 0;
//
//        System.err.println("queue : get : called");
//        while( true ){
//
//            /*
//             *  We check to see if there is element in the queue
//             *  if there is, we return the head
//             *  if not, we wait for 2 sec only once, if there is still
//             *  no element, we return null
//             */
//            synchronized( m_queueMutex ){
//                if( m_size != 0 ){
//                    ojb = m_head.object;
//
//                    if( m_size == 1 ){
//                        m_head = null;
//                        m_tail = null;
//                    }
//                    else{
//                        m_head = m_head.next;
//                        m_head.prev = null;
//                    }
//
//                    m_size--;
//
//                    System.err.println("queue : get : " + ojb.toString() );
//                    return ojb;
//                }
//                else if( nbRetries > 0 )
//                    return null;
//            }
//
//            /*
//             *  If we reach here , it means that m_size was 0
//             *  we wait once to see if there is going to be some additional
//             *  stuff. We wait for 1 second before we return null
//             */
//            try{
//                nbRetries++;
//                wait(1000);
//            }
//            catch( InterruptedException intEx ){
//            }
//        }
//    }
////------------------------------------------------------------------------------
//    public boolean contains( Object obj ){
//        QueueNode node = null;
//
//        synchronized( m_queueMutex ){
//            if( m_size == 0 )
//                return false;
//
//            for( node = m_head; node != null; node = node.next ){
//                if( node.object.equals(obj) )
//                    return true;
//            }
//        }
//
//        return false;
//    }
////------------------------------------------------------------------------------
//    public synchronized void setState( int state ){
//        m_state = state;
//        notifyAll();
//    }
////------------------------------------------------------------------------------
//}
//
////------------------------------------------------------------------------------
////------------------------------------------------------------------------------
////------------------------------------------------------------------------------
//class QueueNode {
//
//    public QueueNode prev;
//    public QueueNode next;
//    public Object object;
//
//    public QueueNode( Object object ){
//        this.object = object;
//        prev = null;
//        next = null;
//    }
//
//    public QueueNode( Object object, QueueNode previous, QueueNode next ){
//        this.object = object;
//        prev = previous;
//        next = next;
//    }
//}
//
////------------------------------------------------------------------------------
////------------------------------------------------------------------------------
////------------------------------------------------------------------------------
////class FileSearcher extends Thread {
////
////    //private BufferedReader m_file;
////    private FileReader m_file;
////
////    FileSearcher( File searchFile ) throws FileNotFoundException {
////        if( searchFile == null )
////            throw( new IllegalArgumentException("searchFile is invalid") );
////
////        m_file = new FileReader(searchFile);
////    }
////
//////------------------------------------------------------------------------------
////    public boolean contains( Object obj ){
////        BufferedReader searchFile = null;
////        String tmpStr = null;
////
////
////        try{
////            synchronized( m_file ){
////                searchFile = new BufferedReader( m_file );
////                while( (tmpStr = searchFile.readLine()) != null ){
////                    if( tmpStr.equals( obj.toString() ) ){
////                        try{
////                            searchFile.close();
////                        }
////                        catch( IOException closeEx ){
////                        }
////
////                        return true;
////                    }
////                }
////            }
////        }
////         catch( IOException ioEx ){
////             if( searchFile != null ){
////                try{
////                    searchFile.close();
////                }
////                catch( IOException closeEx ){
////                }
////             }
////            return false;
////        }
////
////
////        try{
////            searchFile.close();
////        }
////        catch( IOException closeEx ){
////        }
////
////        return false;
////    }
////------------------------------------------------------------------------------
////}
