/*
 ** Copyright (C) 2001,2002 Sacha Faust <sacha@severus.org>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
 ** the Free Software Foundation; either version 2 of the License, or
 ** (at your option) any later version.
 **
 ** This program is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
 ** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ** GNU General Public License for more details.
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program; if not, write to the Free Software
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 *  $Revision: 2 $
 *  $Modtime: 3/18/03 12:39a $
 *  Soft Version : 2.1
 */
package faust.sacha.web.bot.spider.customlogin;

import java.io.*;
import java.net.*;
import java.util.*;

import org.apache.commons.httpclient.*;
import org.apache.commons.httpclient.methods.*;

import faust.sacha.web.data.*;
import faust.sacha.web.util.*;

public class CustomFormLoginCookieLoader extends CustomLoginLoader {
    
    CustomFormLoginCookie m_formLogin;
    
    public CustomFormLoginCookieLoader( CustomLogin login ) {
        super(login);
        
        if( !(login instanceof CustomFormLoginCookie) )
            throw( new IllegalArgumentException("login in invalid") );
        
        m_formLogin = (CustomFormLoginCookie)login;
    }
//------------------------------------------------------------------------------    
    protected boolean processLogin() {
        URLCont urlData = null;
        URLData url = null;
        PostMethod requestMethod = null;
        int requestCode = 0;
        boolean allGood = true;
        int nbTries = 0;
        Header cookieData[] = null;

        try{
            url = new URLData( m_formLogin.getURL() );
        }
        catch( MalformedURLException urlEX ){
            return false;
        }
       
        
        urlData = new URLCont( (URLInfo)url );
        requestMethod = (PostMethod)m_formLogin.getMethod();
        
        prepareRequestMethod( url, requestMethod, WebGlobal.FOLLOW_INSIDE_REDIRECT );

        // get rid of expired cookies
        urlData.purgeCookies();

        while( true ){
            try{
                requestCode = requestMethod.execute(urlData.getState(), urlData);

                /*
                 *  Set the socket Timeout
                 *
                 *  TODO : see if we really need to get out of the while here if something happens
                 */
                try{
                    //making sure it's open
                    if( !urlData.isOpen() )
                        urlData.open();

                    urlData.setSoTimeout(WebGlobal.SOCKET_TIMEOUT);
                }
                catch( IllegalStateException stateEx ){
                    System.err.println( "CustomFormLoginCookieLoader::processLogin() : IllegalStateException : setting socket timeout : " + stateEx );
                }
                catch( SocketException sockEx ){
                    System.err.println( "CustomFormLoginCookieLoader::processLogin() : SocketException : setting socket timeout : " + sockEx );
                }
                catch( IOException ioEx ){
                    System.err.println( "CustomFormLoginCookieLoader::processLogin() : IOException : setting socket timeout : " + ioEx );
                }

                // Checking return code
                switch( requestCode ){
                    case HttpStatus.SC_MOVED_TEMPORARILY:       // 302 Moved Temporarily (Sometimes Found) (HTTP/1.0 - RFC 1945)
                        if( WebGlobal.DEBUG )
                            System.err.println( "CustomFormLoginCookieLoader::processLogin() : got redirect on : " + url.getURL() );

                        allGood = false;
                        break;

                    // ERRORS
                    case HttpStatus.SC_BAD_REQUEST:             //  400 Bad Request (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_NOT_FOUND:               //  404 Not Found (HTTP/1.0 - RFC 1945)

                        // TODO , look if this is really valid.
                        /*
                         *  If the url is a query, we try to fetch the data anyways
                         *  since we don't set allGood to false, it will we send to getFileData()
                         *  because the server might reply 400 or 404 because of the query
                         */
                        //if( url.getQuery().equals("") )
                            allGood = false;
                        break;
                    case HttpStatus.SC_GONE:                    //  410 Gone (HTTP/1.1 - RFC 2616)
                        allGood = false;
                        break;
                    case HttpStatus.SC_NOT_ACCEPTABLE:          //  406 Not Acceptable (HTTP/1.1 - RFC 2616)
                        allGood = false;
                        break;

                    // handling timeouts
                    case HttpStatus.SC_REQUEST_TIMEOUT:         //  408 Request Timeout (HTTP/1.1 - RFC 2616)
                        if( nbTries >= WebGlobal.getTimeoutNbRetries() ){
                            /*
                             * There is no point to continue. We return immediatly
                             */
                            return false;
                        }
                        else{
                            nbTries++;
//                            try{
//                                sleep(100);
//                            }
//                            catch( InterruptedException intEx ){
//                                if( WebGlobal.DEBUG )
//                                System.err.println( "CustomFormLoginCookieLoader::processLogin() : " + intEx );
//                            }
                            continue;
                        }

                    // Interesting errors
                    case HttpStatus.SC_INTERNAL_SERVER_ERROR:   //  500 Server Error (HTTP/1.0 - RFC 1945)
                    case HttpStatus.SC_SERVICE_UNAVAILABLE:     //  503 Service Unavailable (HTTP/1.0 - RFC 1945)
                        System.err.println( "CustomFormLoginCookieLoader::processLogin() : got Server Error " + requestCode + " on : " + url.getURL() );
                        allGood = false;
                        break;

                    // section of error codes that indicate that the page is present but we can't see them
                    case HttpStatus.SC_UNAUTHORIZED:            //  401 Unauthorized (HTTP/1.0 - RFC 1945)
                        /*
                         *  We add the url here since we won't be able to fetch it later
                         */
                        allGood = false;
                        break;

                    // Section of error codes that I want to see if I get them
                    case HttpStatus.SC_PAYMENT_REQUIRED:        //  402 Payment Required (HTTP/1.1 - RFC 2616) : This code is reserved for future use.
                    case HttpStatus.SC_FORBIDDEN:               //  403 Forbidden (HTTP/1.0 - RFC 1945)
                    case HttpStatus.SC_METHOD_NOT_ALLOWED:      //  405 Method Not Allowed (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_CONFLICT:                //  409 Conflict (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_LENGTH_REQUIRED:         //  411 Length Required (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_PRECONDITION_FAILED:     //  412 Precondition Failed (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_REQUEST_TOO_LONG:        //  413 Request Entity Too Large (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_REQUEST_URI_TOO_LONG:    //  414 Request-URI Too Long (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_UNSUPPORTED_MEDIA_TYPE:  //  415 Unsupported Media Type (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_REQUESTED_RANGE_NOT_SATISFIABLE:     //  416 Requested Range Not Satisfiable (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_EXPECTATION_FAILED:      //  417 Expectation Failed (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_INSUFFICIENT_SPACE_ON_RESOURCE:      //  419 Insufficient Space on Resource (WebDAV - draft-ietf-webdav-protocol-05?) or 419 Proxy Reauthentication Required (HTTP/1.1 drafts?)
                    case HttpStatus.SC_METHOD_FAILURE:          //  420 Method Failure (WebDAV - draft-ietf-webdav-protocol-05?)
                    case HttpStatus.SC_UNPROCESSABLE_ENTITY:    //  422 Unprocessable Entity (WebDAV - RFC 2518)
                    case HttpStatus.SC_LOCKED:                  //  423 Locked (WebDAV - RFC 2518)
                    case HttpStatus.SC_FAILED_DEPENDENCY:       //  424 Failed Dependency (WebDAV - RFC 2518)
                    case HttpStatus.SC_NOT_IMPLEMENTED:         //  501 Not Implemented (HTTP/1.0 - RFC 1945)
                    case HttpStatus.SC_HTTP_VERSION_NOT_SUPPORTED:      //  505 HTTP Version Not Supported (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_INSUFFICIENT_STORAGE:    //  507 Insufficient Storage (WebDAV - RFC 2518)
                        System.err.println( "CustomFormLoginCookieLoader::processLogin() : got un usual error " + requestCode + " on : " + url.getURL() );
                        allGood = false;
                        break;

                    // section indicating proxy error
                    case HttpStatus.SC_PROXY_AUTHENTICATION_REQUIRED:   // 407 Proxy Authentication Required (HTTP/1.1 - RFC 2616)
                    case HttpStatus.SC_BAD_GATEWAY:             //  502 Bad Gateway (HTTP/1.0 - RFC 1945)
                    case HttpStatus.SC_GATEWAY_TIMEOUT:         //  504 Gateway Timeout (HTTP/1.1 - RFC 2616)
                        System.err.println( "CustomFormLoginCookieLoader::processLogin() : got proxy error " + requestCode + " on : " + url.getURL() );
                        allGood = false;
                        break;
                }
            }
            catch( HttpException httpEx ){
                System.err.println( "CustomFormLoginCookieLoader::processLogin() : HttpException : " + httpEx );
                allGood = false;
            }
            catch( IOException ioExRequest ){
                if( WebGlobal.DEBUG )
                    System.err.println( "CustomFormLoginCookieLoader::processLogin() : IOException:open() : " + nbTries + " : " + ioExRequest );

                if( nbTries >= WebGlobal.getTimeoutNbRetries() ){
                    /*
                     * There is no point to continue. We return immediatly
                     */
                    return false;
                }
                else{
                    nbTries++;
//                    try{
//                        sleep(100);
//                    }
//                    catch( InterruptedException intEx ){
//                        if( WebGlobal.DEBUG )
//                            System.err.println( "CustomFormLoginCookieLoader::processLogin() : " + intEx );
//                    }
                    continue;
                }
            }

            break;
        }
        
        cookieData = requestMethod.getResponseHeaders();
        
        if( cookieData != null ){
            super.m_state.addCookies( getCookie(cookieData) );
        }
        else
            allGood = false;


        urlData.close();


        return allGood;        
    }
//------------------------------------------------------------------------------    
    public Cookie[] getCookie( Header headers[] ){
        Cookie retVal[] = null;
        Cookie tmpCookies[] = null;
        URLInfo url = null;
        List cookieList = null;
        String cookieData = null;
        ArrayList cookieDataList = null;
        url = m_formLogin.getURL();
        
        cookieDataList = new ArrayList(0);
        
        for( int i = 0; i < headers.length; i++ ){
            
            if( headers[i].getName().equalsIgnoreCase("Set-Cookie") ){
                System.err.println(headers[i].getValue());
                cookieList = m_formLogin.getStateCookieList();
                
                for( int j = 0; j < cookieList.size(); j++ ){
                    cookieData = (String)cookieList.get(j);
                
                    //if( headers[i].getValue().toLowerCase().indexOf(cookieData.toLowerCase()) != -1 ){
                        try{
                            tmpCookies = Cookie.parse(url.getHost(), url.getPort(), url.getPath(),url.isSecure(), headers[i]);
                            
                            for( int cookieDataIndex = 0; cookieDataIndex < tmpCookies.length; cookieDataIndex++ ){
                                cookieDataList.add( (Object)tmpCookies[cookieDataIndex] );
                            }
                        }
                        catch( HttpException httpEx ){
                            if( WebGlobal.DEBUG )
                                System.err.println("CustomFormLoginCookieLoader::getCookie() : " + httpEx);
                        }

                        break;
                    //}
                }
            }
        }
        
        for( int d = 0; d < cookieDataList.size(); d++ ){
            Cookie c = (Cookie)cookieDataList.get(d);
            
            //System.err.println("c->" + d + " = " + c.toString() );
        }
        
        retVal = new Cookie[cookieDataList.size()];
        retVal = (Cookie[])cookieDataList.toArray( (Object[])retVal );
        return retVal;
    }
//------------------------------------------------------------------------------    
    private void prepareRequestMethod( URLData url , HttpMethod requestMethod, boolean redirect ){

        requestMethod.setPath( url.getPath() );

        if( url.getQuery() != "" )
            requestMethod.setQueryString( url.getQuery() );
        
        requestMethod.addRequestHeader("User-Agent", WebGlobal.USER_AGENT);

        requestMethod.setFollowRedirects( redirect );
    }    
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
    public static void main( String args[] ){
        CustomFormLoginCookie c = null;
        CustomFormLoginCookieLoader l = null;
        URLInfo url = null;
        String urlStr = null;
        String configData = null;
        HttpState state = null;
        CustomLoginParser parser = null;
        String configFile = null;
        
        WebGlobal.setDebug(true);
        
        //urlStr = "http://dynamic.macromedia.com";
        urlStr = "";
        configData = "user=sacha@severus.org&pass=123456&popup=1&Submit2.x=33&Submit2.y=14:sess";
        configFile = "e:\\temp\\macromedia.xml";
        
        try{
            url = new URLInfo(urlStr);
        }
        catch( java.net.MalformedURLException urlEx ){
            System.exit(1);
        }
        
        parser = new CustomLoginParser(url, configFile);
        parser.process();
        c = (CustomFormLoginCookie)parser.getCustomLogin();
        c.processConfig();
        //c = new CustomFormLoginCookie(url, configData);        
        l = new CustomFormLoginCookieLoader(c);
        
        if( l.process() ){
            state = l.getHttpState();
            
            System.err.println(state.toString());
        }
    }
}
