/*
** Copyright (C) 2001,2002 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
 *  $Revision: 1 $
 *  $Modtime: 12/29/02 2:57a $
 *  Soft Version : 2.1
 */
package faust.sacha.web.bot.spider.customlogin;

import java.io.*;
import java.net.*;
import java.lang.reflect.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;

import faust.sacha.web.util.*;

public class CustomLoginParser {
    
    private String m_configFile;
    private CustomLogin m_customLogin;
    private URLInfo m_url;
    private String m_className;
    private URLInfo m_site;
    private Node m_customHandlerNode;
    
    public CustomLoginParser( URLInfo site, String configFile ) {
        if( configFile == null )
            throw( new IllegalArgumentException("configFile is invalid") );
        if( site == null )
            throw( new IllegalArgumentException("site is invalid") );
        
        m_configFile = configFile;
        m_site = site;        
        m_url = null;
        m_customLogin = null;
        m_className = null;
        m_customHandlerNode = null;
    }
//------------------------------------------------------------------------------
    public CustomLogin getCustomLogin(){
        return m_customLogin;
    }
//------------------------------------------------------------------------------    
    public URLInfo getURL(){
        return m_url;
    }
//------------------------------------------------------------------------------
    public Node getCustomHandlerNode(){
        return m_customHandlerNode;
    }
//------------------------------------------------------------------------------    
    public void process(){
        StringBuffer sb = new StringBuffer();
        try {
            // Read the entire file into memory
            BufferedReader r = new BufferedReader( new FileReader( m_configFile ) );
            String str = r.readLine();
            while( str != null ) {
                sb.append( str ).append( "\r\n" );
                str = r.readLine();
            }
            r.close();
        }
        catch( Exception e) {
            throw( new IllegalArgumentException("configFile is invalid : " + e) );
        }
        
        parseConfiguration(sb.toString());
        
        createCustomLogin();
    }
//------------------------------------------------------------------------------
    protected void parseConfiguration( String xmlString ){
        InputSource inSource = null;
        DocumentBuilderFactory docBuilder = null;
        DocumentBuilder parser = null;
        Document doc = null;
        Element docElement = null;
        NodeList entryList = null;
        Node entry = null;
        
        try{
            inSource = new InputSource( new StringReader(xmlString) );
            docBuilder = DocumentBuilderFactory.newInstance();
            parser = docBuilder.newDocumentBuilder();
            doc = parser.parse(inSource);
            
            docElement = doc.getDocumentElement();
            entryList = docElement.getChildNodes();
            
            for( int i = 0; i < entryList.getLength(); i++ ){
                entry = entryList.item(i);
                
                if( entry.getNodeName().equals("CODE") )
                    handleCode(entry);
                else if( entry.getNodeName().equals("LOCATION") )
                    handleLocation(entry);
                else if( entry.getNodeName().equals("HANDLERCONFIG") )
                    handleHandlerConfig(entry);
            }
            
        }
        catch( Exception e ){
            System.err.println(e);
        }
    }
//------------------------------------------------------------------------------
    private void handleCode( Node codeNode ){
        NamedNodeMap nodeMap = null;
        Node tmpNode = null;
        String className = null;
        
        try{
            nodeMap = codeNode.getAttributes();
            tmpNode = nodeMap.getNamedItem("class");

            if( tmpNode != null ){
                m_className = tmpNode.getNodeValue().trim();
                
                if( WebGlobal.DEBUG )
                    System.err.println("CustomLoginParser::handleCode() : code = " + m_className);
            }
        }
        catch( Exception e ){
            System.err.println(e);
        }
    }
//------------------------------------------------------------------------------
    private void handleLocation( Node locationNode ){
        NamedNodeMap nodeMap = null;
        Node tmpNode = null;
        
        try{
            nodeMap = locationNode.getAttributes();
            tmpNode = nodeMap.getNamedItem("url");
            
            
            if( tmpNode != null ){
                try{
                    m_url = new URLInfo(tmpNode.getNodeValue().trim(), m_site);
                    
                    if( WebGlobal.DEBUG )
                        System.err.println("CustomLoginParser::handleLocation() : url = " + m_url.toString());                    
                }
                catch( MalformedURLException urlEx ){
                    throw( new IllegalArgumentException("config is invalid : " + urlEx) );
                }
            }
        }
        catch( Exception e ){
            throw( new IllegalArgumentException("config is invalid : " + e) );
        }
    }
//------------------------------------------------------------------------------           
    private void handleHandlerConfig( Node handlerConfigNode ){
        
        m_customHandlerNode = handlerConfigNode.cloneNode(true);        
    }
//------------------------------------------------------------------------------
    private void createCustomLogin(){
        Class customLoginClass = null;
        CustomLogin login = null;
        Constructor cons = null;
        Object consArg[] = null;
        Class consOptions[] = null;
        
        
        try{
            customLoginClass = Class.forName( m_className );
        }
        catch( ClassNotFoundException classEx ){
            throw( new IllegalArgumentException("config is invalid : " + classEx) );
        }


        /*
         *  we are looking for the template constructor with ( URLInfo url, Node config )
         */
        consOptions = new Class[2];
        consOptions[0] = (Class)URLInfo.class;
        consOptions[1] = (Class)Node.class;

        try{
            cons = customLoginClass.getConstructor(consOptions);
        }
        catch( NoSuchMethodException notThereEx ){
            throw( new IllegalArgumentException("config is invalid : " + notThereEx) );
        }
        catch( SecurityException secEx ){
            throw( new IllegalArgumentException("config is invalid : " + secEx) );
        }

        /*
         * Let's create the new class instance using the constructor we found
         *  From javadoc on newInstance()
         *  ...values of primitive types are wrapped in a wrapper object of the appropriate type (e.g. a float in a Float)
         */
        consArg = new Object[2];
        consArg[0] = (Object)m_url;
        consArg[1] = (Object)m_customHandlerNode;

        try{
            login = (CustomLogin)cons.newInstance( consArg );
        }
        catch( InstantiationException instEx ){
            throw( new IllegalArgumentException("config is invalid : " + instEx) );
        }
        catch( IllegalAccessException illEx ){
            throw( new IllegalArgumentException("config is invalid : " + illEx) );
        }
        catch( IllegalArgumentException illArgEx ){
            throw( new IllegalArgumentException("config is invalid : " + illArgEx) );
        }
        catch( InvocationTargetException zoolooEx ){
            throw( new IllegalArgumentException("config is invalid : " + zoolooEx) );
        }

        m_customLogin = login;
    }
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
    public static void main( String args[] ){
        String fileName = null;
        CustomLoginParser p = null;
        URLInfo url = null;
        
        WebGlobal.setDebug(true);
        
        fileName = "O:\\Java\\Misc\\faust\\sacha\\testing\\TESTXML.xml";
        try{
            url = new URLInfo("http://www.domain.com/");
        }
        catch( MalformedURLException urlEx ){
        }
        
        p = new CustomLoginParser(url ,fileName);
        
        p.process();
        
    }
}
