/*
** Copyright (C) 2001,2002, 2003 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
 *  $Revision: 5 $
 *  $Modtime: 3/19/03 12:21a $
 *  Soft Version : 2.1
 */

// idea and suggestions from Jesse Wilson <jesse.wilson@bmo.com>
package faust.sacha.web.data;


import java.util.*;
import java.io.*;
import java.lang.reflect.*;

import faust.sacha.web.util.*;

public class DataFactory {

    public static synchronized List createList(){
        if( WebGlobal.USE_FILE_IO_FOR_SPIDER_DATA )
            try{
                return getSiteElementInfoHandler();
            }
            catch( Exception ex ){      // any exception that might be thrown by a custom class written by the user
                System.err.println( "DataFactory::createList() : got exception by custom class : " + WebGlobal.getSiteElementHandler() + " : exception : " + ex );
                ex.printStackTrace();
                return null;
            }

        else
            return (List)new ArrayList();
    }
//------------------------------------------------------------------------------
    public static List cloneList(Collection c){
        return (List)new ArrayList(c);
    }
//------------------------------------------------------------------------------
    private static List getSiteElementInfoHandler(){
        List handler = null;
        Constructor cons = null;
        Class loaderClass = null;
        Class consClassType[] = null;
        Object consArg[] = null;
        String loaderName = null;

        loaderName = WebGlobal.getSiteElementHandler();

        try{
            loaderClass = Class.forName(loaderName, true, ClassLoader.getSystemClassLoader());
        }
        catch( ClassNotFoundException classEx ){
            System.err.println("DataFactory::getSiteElementInfoHandler() : unable to load class : " + loaderName + " | " + classEx );
            return null;
        }

        /*
         *  We are looking for : list() constructor
         */

        consClassType = new Class[0];

        try{
            cons = loaderClass.getConstructor(consClassType);
        }
        catch( NoSuchMethodException methodEx ){
            System.err.println("DataFactory::getSiteElementInfoHandler() : unable to find constructor for : " + loaderName + " | " + methodEx);

            return null;
        }
        catch( SecurityException secEx ){
            System.err.println("DataFactory::getSiteElementInfoHandler() : security exception for : " + loaderName + " | " + secEx);

            return null;
        }

        /*
         *  Let's call the constructor and get a new instance
         */
        consArg = new Object[0];

        try{
            handler = (List)cons.newInstance(consArg);      // if it's not a class implementing List they will get the classcast exception and I let them deal with that
        }
        catch( InstantiationException instEx ){
            System.err.println("DataFactory::getSiteElementInfoHandler() : InstantiationException exception for : " + loaderName + " | " + instEx );
            return null;
        }
        catch( IllegalAccessException illEx ){
            System.err.println("DataFactory::getSiteElementInfoHandler() : IllegalAccessExceptionexception for : " + loaderName + " | " + illEx );
            return null;
        }
        catch( IllegalArgumentException illArgEx ){
            System.err.println("DataFactory::getSiteElementInfoHandler() : IllegalArgumentExceptionfor : " + loaderName + " | " + illArgEx );
            return null;
        }
        catch( InvocationTargetException zoolooEx ){
            System.err.println("DataFactory::getSiteElementInfoHandler() : InvocationTargetException : " + loaderName + " | " + zoolooEx );
            return null;
        }


        return handler;
    }
}
