/*
** Copyright (C) 2001,2002, 2003 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
 *  $Revision: 32 $
 *  $Modtime: 3/19/03 12:21a $
 *  Soft Version : 2.1
 */
package faust.sacha.web.data;

import faust.sacha.web.data.*;
import faust.sacha.web.util.*;
import java.net.*;
import java.util.*;
import java.lang.reflect.*;

import org.apache.commons.httpclient.*;

import faust.sacha.web.bot.spider.*;
import faust.sacha.web.bot.spider.event.*;
import faust.sacha.web.bot.spider.data.*;
import faust.sacha.web.bot.spider.customlogin.*;
import faust.sacha.web.bot.spider.state.*;

public class Site extends ThreadEventManager{

    private Hashtable m_siteFolders;
    private URLInfo m_urlInfo;
    private ProcessBroker m_broker;
    private List m_urlsDone;
    private WaitUrlQ m_todoURLS;
    private UrlQ m_toGetDataURLS;
    private UrlQ m_toGetFlashURLS;
    private List m_emails;
    private List m_outsideURLs;
    private GetInfoManager m_getInfoManager;
    private GetDataManager m_getDataManager;
    private GetFlashManager m_getFlashManager;
    private SiteURLProcessor m_urlProcessor;
    private SpiderManager m_spiderManager;
    private String m_emailDomain;
    private Hashtable m_keywordList;

    public Site( URLInfo url, SpiderManager manager, CustomLogin customLogin ) throws MalformedURLException{
        super( "Site->" + url.getHost() );

        m_urlInfo = url;

        m_siteFolders = new Hashtable();
        m_urlsDone = DataFactory.createList();
        m_emails = DataFactory.createList();
        m_outsideURLs = DataFactory.createList();

        m_keywordList = new Hashtable();

        m_toGetDataURLS = new UrlQ();
        m_todoURLS = new WaitUrlQ( "WaitUrlQ->" + getDomain(), this, m_toGetDataURLS );
        m_toGetFlashURLS = new UrlQ();

        m_urlProcessor = new SiteURLProcessor("SiteURLProcessor->" + getDomain(), this, m_todoURLS);

        m_broker = new ProcessBroker( "ProcessBroker->" + getDomain(), m_toGetDataURLS, m_todoURLS, m_toGetFlashURLS, m_urlProcessor, this );
        m_getInfoManager = new GetInfoManager( "GetInfoManager->" + getDomain(), m_todoURLS, m_broker );
        m_getDataManager = new GetDataManager( "GetDataManager->" + getDomain(), m_toGetDataURLS, m_broker );
        m_getFlashManager = new GetFlashManager( "GetFlashManager->" + getDomain(), m_toGetFlashURLS, m_broker );

        m_spiderManager = manager;

        /*
         *  Add the first url todo in the queue
         */
        m_todoURLS.add( new URLData(m_urlInfo) );

        /*
         *  We add all the default urls to do
         */
         addDefaultURLsToSpider();

         registerHttpState(customLogin);
    }

    public Site( String url, SpiderManager manager, CustomLogin customLogin ) throws MalformedURLException{
        this( new URLInfo(url), manager, customLogin );
    }

    public Site( URLInfo url, String emailDomain, SpiderManager manager, CustomLogin customLogin ) throws MalformedURLException{
        this( url, manager, customLogin );
        m_emailDomain = emailDomain;
    }

    public Site( String url, String emailDomain, SpiderManager manager, CustomLogin customLogin ) throws MalformedURLException{
        this( new URLInfo(url), manager, customLogin );
        m_emailDomain = emailDomain;
    }
//------------------------------------------------------------------------------
    private void registerHttpState( CustomLogin customLogin ){
        HttpState state = null;
        CustomLoginLoader loader = null;
        String loaderClass = null;

        if( customLogin != null )
            loader = getCustomLoginLoader( customLogin );

        if( loader != null ){
            if( loader.process() )
                state = loader.getHttpState();
        }

        if( state == null )
            state = new HttpState();

        // register the state to the state manager
        HttpStateManager.addSiteState(m_urlInfo.getHost(), state);
    }
//------------------------------------------------------------------------------
    private CustomLoginLoader getCustomLoginLoader( CustomLogin customLogin ){
        Class customLoginLoaderClass = null;
        CustomLoginLoader loader = null;
        Constructor cons = null;
        Object consArg[] = null;
        Class consOptions[] = null;


        try{
            customLoginLoaderClass = Class.forName( customLogin.getLoaderName() );
        }
        catch( ClassNotFoundException classEx ){
            System.err.println("Site::getCustomLoginLoader() : " + classEx );
            return null;
        }


        /*
         *  we are looking for the template constructor with ( CustomLogin )
         */
        consOptions = new Class[1];
        consOptions[0] = (Class)CustomLogin.class;

        try{
            cons = customLoginLoaderClass.getConstructor(consOptions);
        }
        catch( NoSuchMethodException notThereEx ){
             System.err.println("Site::getCustomLoginLoader() : " + notThereEx );
        }
        catch( SecurityException secEx ){
            System.err.println("Site::getCustomLoginLoader() : " + secEx );
        }

        /*
         * Let's create the new class instance using the constructor we found
         *  From javadoc on newInstance()
         *  ...values of primitive types are wrapped in a wrapper object of the appropriate type (e.g. a float in a Float)
         */
        consArg = new Object[1];
        consArg[0] = (Object)customLogin;

        try{
            loader = (CustomLoginLoader)cons.newInstance( consArg );
        }
        catch( InstantiationException instEx ){
            System.err.println("Site::getCustomLoginLoader() : " + instEx );
        }
        catch( IllegalAccessException illEx ){
            System.err.println("Site::getCustomLoginLoader() : " + illEx );
        }
        catch( IllegalArgumentException illArgEx ){
            System.err.println("Site::getCustomLoginLoader() : " + illArgEx );
        }
        catch( InvocationTargetException zoolooEx ){
            System.err.println("Site::getCustomLoginLoader() : " + zoolooEx );
        }

        return loader;
    }
//------------------------------------------------------------------------------
    public void run(){
        super.run();

        addToListeners(m_broker);
        addToListeners(m_getDataManager);
        addToListeners(m_getInfoManager);
        addToListeners(m_getFlashManager);
        addToListeners(m_urlProcessor);
        addToListeners(m_todoURLS);

        process();

        if( WebGlobal.DEBUG ){
            System.err.println(getName() + "->After process()");
            System.err.println(getName() + "->END RUN()");
        }
    }
//------------------------------------------------------------------------------
    private void addDefaultURLsToSpider(){

        String tmpStr = null;
        Vector fileList = null;
        Vector folderList = null;

        // No ulrs to add
        if( WebGlobal.URLS_TO_INCLUDE == null )
            return;

        fileList = (Vector)WebGlobal.URLS_TO_INCLUDE.get("file");
        folderList = (Vector)WebGlobal.URLS_TO_INCLUDE.get("folder");

        // files
        for( int i = 0; i < fileList.size(); i++ ){
            tmpStr = (String)fileList.get(i);

            if( tmpStr.equals("") )
                continue;

            try{
                m_todoURLS.add( new URLData(tmpStr, m_urlInfo, WebGlobal.URL_TYPE_FILE) );
            }
            catch( MalformedURLException urlEx ){
                if( WebGlobal.DEBUG )
                    System.err.println( getName() + "::addDefaultURLsToSpider() : " + urlEx );
            }
        }

        // folders
        for( int i = 0; i < folderList.size(); i++ ){
            tmpStr = (String)folderList.get(i);

            try{
                m_todoURLS.add( new URLData(tmpStr, m_urlInfo, WebGlobal.URL_TYPE_FOLDER) );
            }
            catch( MalformedURLException urlEx ){
                if( WebGlobal.DEBUG )
                    System.err.println( getName() + "::addDefaultURLsToSpider() : " + urlEx );
            }
        }
    }
//------------------------------------------------------------------------------
    private synchronized void process(){
        ThreadEvent event = null;
        ThreadEvent eventRemove = null;
        int state = 0;
        int nbSiteSpiderLoop = 0;

        m_broker.start();
        m_getInfoManager.start();
        m_getDataManager.start();
        m_getFlashManager.start();
        m_urlProcessor.start();
        m_todoURLS.start();

        while( true ){
            state = getState();

            if( (state == ThreadEvent.EVENT_STOP) || (state == ThreadEvent.EVENT_SITE_FINISH ) )
                break;

            if( state == ThreadEvent.EVENT_IS_SITE_FINISH ){
                if( WebGlobal.DEBUG )
                    System.err.println( getName() + "::run() entering EVENT_IS_SITE_FINISH" );

                /*
                 *  This is a protection agains sites that will return crap info when
                 *  HEAD a basic HEAD request is sent on the /.
                 *  By counting the number of times the spider did a loop and the number of
                 *  urls done by the spider, we can see if such event might have happened and
                 *  we force the root url of the site to be processed got a GET request
                 *
                 *  Example of such site:
                 *      www.sharp-usa.com [206.65.179.140] 80 (http) open
                 *      HEAD / HTTP/1.0
                 *
                 *      HTTP/1.1 302 Object moved
                 *      Server: Microsoft-IIS/5.0
                 *      Date: Sun, 31 Mar 2002 02:01:22 GMT
                 *      Location: http://www.sharp-usa.com
                 *      Connection: Keep-Alive
                 *      Content-Length: 145
                 *      Content-Type: text/html
                 *      Set-Cookie: ASPSESSIONIDGQQGQCBG=IHJGOOABMHKDIIEILLLAEPGA; path=/
                 *      Cache-control: private
                 */
                if( nbSiteSpiderLoop == 0 ){
                    if( m_urlsDone.size() == 1 ){
                        try{
                            m_toGetDataURLS.add( new URLData(m_urlInfo) );
                        }
                        catch( MalformedURLException urlEx ){
                        }
                    }

                }
                nbSiteSpiderLoop++;

                /*
                 *  Wait for both m_broker and m_urlProcessor to finish
                 *  their processing. This will enfore the other queues
                 *  to get filled with data if some is available after they
                 *  finish processing
                 */
                m_broker.waitForProcessing();
                m_urlProcessor.waitForProcessing();

                if( WebGlobal.DEBUG )
                    System.err.println( getName() + "::run() entering EVENT_IS_SITE_FINISH-2" );

                if( !m_todoURLS.isEmpty() ){
                    if( WebGlobal.DEBUG )
                        System.err.println(getName() + "::process : m_todoURLS is not empty");
                    setState(ThreadEvent.EVENT_NEW);
                }
                else if( !m_toGetDataURLS.isEmpty() ){
                    if( WebGlobal.DEBUG )
                        System.err.println(getName() + "::process : m_toGetDataURLS is not empty");
                    setState(ThreadEvent.EVENT_NEW);
                }
                else if( !m_toGetFlashURLS.isEmpty() ){
                    if( WebGlobal.DEBUG )
                        System.err.println(getName() + "::process : m_toGetFlashURLS is not empty");
                    setState(ThreadEvent.EVENT_NEW);
                }
                else if( m_getFlashManager.getNbThreads() != 0 ){
                    if( WebGlobal.DEBUG )
                        System.err.println(getName() + "::process : still threads in m_getFlashManager : " + m_getFlashManager.getNbThreads());

                    setState(ThreadEvent.EVENT_NEW);
                }
                else if( m_getInfoManager.getNbThreads() != 0 ){
                    if( WebGlobal.DEBUG )
                        System.err.println(getName() + "::process : still threads in m_getInfoManager : " + m_getInfoManager.getNbThreads());
                    setState(ThreadEvent.EVENT_NEW);
                }
                else if( m_getDataManager.getNbThreads() != 0){
                    if( WebGlobal.DEBUG )
                        System.err.println(getName() + "::process : still threads in m_getDataManager : " + m_getDataManager.getNbThreads());
                    setState(ThreadEvent.EVENT_NEW);
                }
                else{
                    setState(ThreadEvent.EVENT_SITE_FINISH);
                    break;
                    //continue;
                }
            }
            else{
                try{
                    /*
                     *  We release the lock and wait to be notified
                     */
                    wait();
                }
                catch( InterruptedException intEx ){
                    System.err.println( getName() + "::run() : " + intEx);
                }
            }
        }

        state = getState();
        if( state == ThreadEvent.EVENT_STOP ){
            stopAllListenningThreads();
        }
        else if( state == ThreadEvent.EVENT_SITE_FINISH ){
            event = new ThreadEvent(this, null, ThreadEvent.EVENT_SITE_FINISH);
            sendEventAll(event);
        }

        /*
         *  Tell the SpiderManager that we are done
         */
        eventRemove = new ThreadEvent(this, null, ThreadEvent.EVENT_FINISH);
        sendEvent(eventRemove, m_spiderManager);
    }
//------------------------------------------------------------------------------
    public WaitUrlQ DEBUG_getTODO(){
        return m_todoURLS;
    }
//------------------------------------------------------------------------------
    protected boolean isEmailAlreadyFound( EmailURL emailToCheck ){
        synchronized( m_emails ){
            return m_emails.contains( (Object)emailToCheck.getEmailURL() );
        }
    }
//------------------------------------------------------------------------------
    protected boolean isFileAlreadyPresent( ArrayList elementList, String fileName ){
        FileURL fileIndex = null;

        synchronized( elementList ){
            for( int i = 0; i < elementList.size(); i++ ){
                fileIndex = (FileURL)elementList.get(i);

                synchronized( fileIndex ){
                    if( fileIndex.getFileName().compareToIgnoreCase(fileName) == 0 )
                        return true;
                }
            }
        }

        return false;
    }
//------------------------------------------------------------------------------
    private void updateFileQueryData( String folder, FileURL fileData ){
        ArrayList fileList = null;
        FileURL fileToCheck = null;
        String queryToAdd = null;

        synchronized( fileData ){
            fileList = getFolderElements(folder);

            synchronized( fileList ){
                queryToAdd = fileData.getQuery();

                if( queryToAdd == "" )
                    return;
                /*
                 *  we go over each files in the folder and compare only
                 *  the filename to see if there is a match.
                 *  if we find a match , we add the queryToAdd to the file
                 */

                for( int i = 0; i < fileList.size(); i++ ){
                    fileToCheck = (FileURL)fileList.get(i);

                    synchronized( fileToCheck ){
                        // it's safe to only compare filenames
                        if( fileToCheck.getFileName().compareToIgnoreCase(fileData.getFileName()) == 0 ){
                            fileToCheck.addQueryData( fileData );
                        }
                    }
                }
            }
        }
    }
//------------------------------------------------------------------------------
    public ArrayList getFolderElements( String folderName ){
        synchronized( m_siteFolders ){
            return (ArrayList)m_siteFolders.get(folderName);
        }
    }
//------------------------------------------------------------------------------
    public String getSiteURL(){
        return m_urlInfo.getBaseURL();
    }
//------------------------------------------------------------------------------
    public int getPort(){
        return m_urlInfo.getPort();
    }
//------------------------------------------------------------------------------
    public boolean isSecure(){
        return m_urlInfo.isSecure();
    }
//------------------------------------------------------------------------------
    public String getDomain(){
        return m_urlInfo.getHost();
    }
//------------------------------------------------------------------------------
    public void addFolder( String folderName ){

        synchronized( m_siteFolders ){
            if( !m_siteFolders.containsKey((Object)folderName) ){
                m_siteFolders.put( folderName, (Object)new ArrayList(0) );
                if( WebGlobal.DEBUG )
                    System.err.println( getName() + "::addFolder() : Adding folderKey " + folderName );
            }
            else{
                if( WebGlobal.DEBUG )
                    System.err.println(getName() + "::addFolder() : duplicate key " + folderName);
            }
        }
    }
//------------------------------------------------------------------------------
    public void addFileToFolder( String folderName, FileURL fileToAdd ){
        ArrayList folderElements = null;
        FileURL presentFile = null;
        String queryToAdd = null;
        ArrayList fileList = null;

        /*
         *  if there is files in the folder
         *      we check if the file is already present in the folder
         *      if the file is present, we try to add new query data to it
         *      else we add the file to the folder
         *  else we try to add the folder and call back the same method
         */
        synchronized( m_siteFolders ){
            if( (folderElements = getFolderElements(folderName)) != null ){
                if( !isFileAlreadyPresent(folderElements, fileToAdd.getFileName()) )
                    folderElements.add( (Object)fileToAdd );
                else{
                    // if the file already exist, try to add some query data to it
                    updateFileQueryData(folderName, fileToAdd);
                }
            }
            else{
                //the folder doesn't exist yet. create it and callback the same function
                if( WebGlobal.DEBUG )
                    System.err.println(getName() + "::addFileToFolder() : folder wasn't present and adding it for : " + folderName );

                addFolder(folderName);
                addFileToFolder(folderName, fileToAdd);
            }
        }
    }
//------------------------------------------------------------------------------
    public void addURLDone( URLInfo urlToAdd ){
        /*
         *  We only add the url without the anchor
         */

        m_urlsDone.add( (Object)urlToAdd.getURLWithoutAnchor() );

        if( WebGlobal.DEBUG )
            System.err.println( getName() + "::addURLDone : added : " + urlToAdd.getURL() );
    }
//------------------------------------------------------------------------------
    public void addEmail( EmailURL emailToAdd ){
        m_emails.add( (Object)emailToAdd.getEmailURL() );
    }
//------------------------------------------------------------------------------
    public List getUrlsDone(){
        return m_urlsDone;
    }
//------------------------------------------------------------------------------
    public List getEmailsFound() {
        return m_emails;
    }
//------------------------------------------------------------------------------
    public boolean isURLAlreadyDone( URLInfo urlToCheck ){

        return m_urlsDone.contains( (Object)urlToCheck.getURLWithoutAnchor() );
    }
//------------------------------------------------------------------------------
    public URLInfo getInfo(){
        return m_urlInfo;
    }
//------------------------------------------------------------------------------
    public Hashtable getFolderHashTable(){
        return m_siteFolders;
    }
//------------------------------------------------------------------------------
    public Enumeration getFoldersNames() {
        return m_siteFolders.keys();
    }
//------------------------------------------------------------------------------
    public void addOutsideURL( URLInfo url ){

        m_outsideURLs.add( (Object)url.getURLWithoutAnchor() );
    }
//------------------------------------------------------------------------------
    public boolean isOutsideURLPresent( URLInfo urlToTest ){
        return m_outsideURLs.contains( urlToTest.getURLWithoutAnchor() );
    }
//------------------------------------------------------------------------------
    public List getOutsideURLs(){
        return m_outsideURLs;
    }
//------------------------------------------------------------------------------
    public String getEmailDomain(){
        return m_emailDomain;
    }
//------------------------------------------------------------------------------
    public void setKeywordFound( String keyword, String url ){
        Vector urlList = null;

        if( WebGlobal.DEBUG )
            System.err.println(getName() + "::setKeywordFound : keyword = " + keyword + " | url = " + url );

        if( m_keywordList.containsKey((Object)keyword) ){
            urlList = (Vector)m_keywordList.get( (Object)keyword );
            urlList.add( (Object)url );
        }
        else
            m_keywordList.put( (Object)keyword, new Vector() );
    }
//------------------------------------------------------------------------------
    public Enumeration getKeywordKeys(){
        return m_keywordList.keys();
    }
//------------------------------------------------------------------------------
    public List getKeywordURL( String keyword ){
        return (List)m_keywordList.get( (Object)keyword );
    }
//------------------------------------------------------------------------------
    public void receiveEvent( ThreadEvent event ){
        Object result = null;
        Object source = null;
        int eventType = 0;

        result = event.getResult();
        source = event.getSource();
        eventType = event.getEventType();

        switch( eventType ){
            case ThreadEvent.EVENT_ADD_URL_TO_SPIDER:
                if( result == null ){
                    if( WebGlobal.DEBUG )
                        System.err.println( getName() + "::receiveEvent : got EVENT_ADD_URL_TO_SPIDER and result was null" );

                    break;
                }
                else if( !(result instanceof String) ){
                    if( WebGlobal.DEBUG )
                        System.err.println( getName() + "::receiveEvent : got EVENT_ADD_URL_TO_SPIDER and result was not String : " + result.getClass().getName() );

                    break;
                }
                else{
                    try{
                        m_todoURLS.add( new URLData((String)result, m_urlInfo) );
                    }
                    catch( MalformedURLException urlEx ){
                        if( WebGlobal.DEBUG )
                            System.err.println( getName() + "::receiveEvent : got EVENT_ADD_URL_TO_SPIDER and result is no good : " + urlEx );
                    }
                    break;
                }
            default:
                super.receiveEvent(event);
                break;
        }
    }
//------------------------------------------------------------------------------
}
