/*
** Copyright (C) 2001,2002, 2003 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
 *  $Revision: 3 $
 *  $Modtime: 3/19/03 12:21a $
 *  Soft Version : 2.1
 */
package faust.sacha.web.util;

import java.io.*;
import java.util.*;
import java.util.regex.*;

import com.iv.flash.parser.*;
import com.iv.flash.api.*;
import com.iv.flash.util.*;
import com.iv.flash.api.action.*;
import com.iv.flash.api.button.*;

public class FlashFileInfoGetter {

    private Pattern protocolPattern = Pattern.compile(WebGlobal.REGEX_PROTO_PATTERN);    // find protocol pattern at the beginning of the script

    private InputStream m_stream;
    private List m_urlList;
    private List m_emailList;
    private List m_miscLinks;

    public FlashFileInfoGetter( InputStream stream ) {
        if( stream == null )
            throw( new IllegalArgumentException("stream is invalid") );

        m_stream = stream;

        // TODO : use Datafactory
        m_urlList = (List)new ArrayList(0);
        m_emailList = (List)new ArrayList(0);
        m_miscLinks = (List)new ArrayList(0);
    }
//------------------------------------------------------------------------------
    public void process(){
        FlashBuffer buff = null;
        FlashFile fFile = null;
        Program prg = null;
        Enumeration descEnum = null;
        Parser p = null;
        Script sc = null;
        Button2 bt2 = null;
        Button bt = null;
        IVVector conVec = null;
        Enumeration conEnum = null;
        Program prog = null;
        Vector urlList = null;
        Script script = null;
        Frame frame = null;
        Timeline timeLine = null;

        //System.err.println("Processing stream");

        try{
            buff = new FlashBuffer( m_stream );
        }
        catch( FileNotFoundException notFoundEx ){
            System.err.println(notFoundEx);
        }
        catch( IOException ioEx ){
            System.err.println(ioEx);
        }

        fFile = FlashFile.newFlashFile();
        p = new Parser();


        try{
            p.parseBuffer(buff, fFile);
        }
        catch( IVException ivEx ){
            if( WebGlobal.DEBUG )
                System.err.println("FlashFileInfoGetter::process() : IVException : " + ivEx );
        }
        catch( Exception e ){
            if( WebGlobal.DEBUG )
                System.err.println("FlashFileInfoGetter::process() : Exception : " + e );
        }

        //System.err.println("Done Processing stream");

        script = fFile.getMainScript();

        if( script != null ){
            timeLine = script.getTimeline();
            for( int i = 0; i < timeLine.getFrameCount(); i++ ){
                frame = timeLine.getFrameAt(i);
                for( int frameIndex = 0; frameIndex < frame.size(); frameIndex++ ){
                    FlashObject fObj = (FlashObject)frame.getFlashObjectAt(frameIndex);

                    if( fObj instanceof DoAction ){
                        DoAction action = (DoAction)fObj;
                        prog = action.getProgram();

                        urlList = prog.getURLS();

                        for( int urlIndex = 0; urlIndex < urlList.size(); urlIndex++ ){
                            FlashURLRef url = (FlashURLRef)urlList.get(urlIndex);
                            addURL( url.getURL() );
                        }
                    }
                }
            }
        }


        descEnum = fFile.definitions();

        if( descEnum == null )
            return;

        while( descEnum.hasMoreElements() ){
            Object o = descEnum.nextElement();

            //----------------------------------------------------
            // Buttons
            if( (o instanceof Button2) || (o instanceof Button) ){
                bt = (Button)o;
                conVec = bt.getActionConditions();

                conEnum = conVec.elements();

                while( conEnum.hasMoreElements() ){
                    Object conOb = conEnum.nextElement();
                    ActionCondition aC = (ActionCondition)conOb;

                    prog = aC.getProgram();

                    urlList = prog.getURLS();

                    for( int i = 0; i < urlList.size(); i++ ){
                        FlashURLRef url = (FlashURLRef)urlList.get(i);
                        addURL( url.getURL() );
                    }
                }
            }
            //----------------------------------------------------
            else if( o instanceof Script ){
                script = (Script)o;
                timeLine = script.getTimeline();

                for( int i = 0; i < timeLine.getFrameCount(); i++ ){
                    frame = timeLine.getFrameAt(i);
                    for( int frameIndex = 0; frameIndex < frame.size(); frameIndex++ ){
                        FlashObject fObj = (FlashObject)frame.getFlashObjectAt(frameIndex);

                        if( fObj instanceof DoAction ){
                            DoAction action = (DoAction)fObj;
                            prog = action.getProgram();

                            urlList = prog.getURLS();


                            for( int urlIndex = 0; urlIndex < urlList.size(); urlIndex++ ){
                                FlashURLRef url = (FlashURLRef)urlList.get(urlIndex);
                                addURL( url.getURL() );
                            }
                        }
                    }
                }
            }
            else{
                //System.err.println("object = " + o.getClass().getName() );
            }
        }
    }
//------------------------------------------------------------------------------
    public List getURLList(){
        return m_urlList;
    }
//------------------------------------------------------------------------------
    public List getEmailList(){
        return m_emailList;
    }
//------------------------------------------------------------------------------
    public List getMiscList(){
        return m_miscLinks;
    }
//------------------------------------------------------------------------------
    private void addURL( String url ){
        Matcher patternMatcher = null;
        String protocolName = null;

        if( url.toLowerCase().startsWith("mailto:") ){
            if( WebGlobal.DEBUG )
                System.err.println("FlashFileInfoGetter::addURL() : adding e-mail : " + url );

            m_emailList.add( (Object)url );
        }
        else{
            patternMatcher = protocolPattern.matcher(url);

            if( patternMatcher.find() ){
                protocolName = patternMatcher.group(1);
                protocolName = protocolName.toLowerCase();

                // not http urls
                if( !(protocolName.startsWith("http")) && !(protocolName.startsWith("https")) ){
                    if( WebGlobal.DEBUG )
                        System.err.println( "FlashFileInfoGetter::addURL() : adding misc link : " + url );

                    m_miscLinks.add( (Object)url) ;
                }
            }
            //ok link
            if( WebGlobal.DEBUG )
                System.err.println("FlashFileInfoGetter::addURL() : adding link : " + url );

            m_urlList.add( (Object)url );
        }
    }
//------------------------------------------------------------------------------


}
