/*
** Copyright (C) 2001,2002 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
 *  $Revision: 2 $
 *  $Modtime: 11/22/02 11:56p $
 *  Soft Version : 2.1
 */
package faust.sacha.web.util;

import java.util.*;

public final class FlashURLRef {

    public static final int METHOD_NONE     =   0;
    public static final int METHOD_GET      =   1;
    public static final int METHOD_POST     =   2;

    private String m_url;
    private String m_target;
    private int m_method;
    private Hashtable m_varDataHash;

    public FlashURLRef(){
        m_url = "";
        m_target = "";
        m_method = METHOD_NONE;
        m_varDataHash = new Hashtable();
    }

    public FlashURLRef( String url, String target, int method ) {
        super();

        if( (url == null) || url.equals("") )
            throw( new IllegalArgumentException("url is invalid") );

        m_url = url;

        /*
         *  target can be null
         */
        if( target != null )
            m_target = "";

        /*
         *  if method is not 1 or 2
         *  we set it to 0 to indicate that none was specified
         */
        if( (method == 1) || (method == 2) )
            m_method = method;
    }
//------------------------------------------------------------------------------
    public void setURL( String url ){
        if( (url != null) && !url.equals("") )
            m_url = url;
    }
//------------------------------------------------------------------------------
    public String getURL(){
        return m_url;
    }
//------------------------------------------------------------------------------
    public void setTarget( String target ){
        if( (target != null) && !target.equals("") )
            m_target = target;
    }
//------------------------------------------------------------------------------
    public String getTarget(){
        return m_target;
    }
//------------------------------------------------------------------------------
    public void setMethod( int method ){
        if( (method != 1) && (method != 2) )
            m_method = method;
        else
            m_method = METHOD_NONE;
    }
//------------------------------------------------------------------------------
    public int getMethod(){
        return m_method;
    }
//------------------------------------------------------------------------------
    public void addVarData( String varName, String data  ){
        List varDataList = null;

        if( (varName == null) || (data == null) )
            return;


        if( !m_varDataHash.containsKey( (Object)varName.toLowerCase() ) ){
            m_varDataHash.put( (Object)varName.toLowerCase(), (Object)new ArrayList(0) );
        }
        else{
            varDataList = (List)m_varDataHash.get( (Object)varName.toLowerCase() );
            varDataList.add( (Object)data );
        }
    }
//------------------------------------------------------------------------------
    public Enumeration getVarList(){
        return m_varDataHash.keys();
    }
//------------------------------------------------------------------------------
    public List getVarDataList( String varName ){
        List varDataList = null;

        if( varName == null )
            return null;

        varDataList = (List)m_varDataHash.get( (Object)varName.toLowerCase() );

        return varDataList;
    }
//------------------------------------------------------------------------------
}
