/*
** Copyright (C) 2001,2002,2003 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
 *  $Revision: 24 $
 *  $Modtime: 3/19/03 12:18a $
 *  Soft Version : 2.1
 */
package org.ideahamster.metis;

import faust.sacha.web.bot.spider.*;
import faust.sacha.web.bot.spider.event.*;
import faust.sacha.web.bot.spider.util.EnginGlobal;
import faust.sacha.web.data.*;
import faust.sacha.web.util.*;

import org.ideahamster.metis.stat.*;
import org.ideahamster.metis.report.*;
import org.ideahamster.metis.util.*;

import gnu.getopt.Getopt;
import java.net.MalformedURLException;

import java.util.*;

/** This is the main file for Metis.
 * @author Sacha Faust
 * @version <PRE>2.1</PRE>
 */
public class Metis extends ThreadEventManager {

    private SpiderManager m_spiderManager;
    private SiteStats m_siteStats;
    private KeyboardProcessor m_keyboardProcessor;

//------------------------------------------------------------------------------
    public Metis() throws MalformedURLException {
        super("Metis");
        Vector siteList = null;

        siteList = MetisOpt.getSiteList();
        if( (siteList == null) || (siteList.size() <= 0) ){
            System.err.println("Unable to get sites from options!");
            System.exit(1);
        }

        m_spiderManager = new SpiderManager( siteList );
        m_keyboardProcessor = new KeyboardProcessor( System.in, m_spiderManager );

        SpiderManager.PRINT_SITE_START_MESSAGE = true;
        SpiderManager.PRINT_SITE_FINISH_MESSAGE = true;

    }
//------------------------------------------------------------------------------
    public void run(){
        Site site = null;
        Enumeration keys = null;
        String siteName = null;
        SiteStats siteStats = null;
        m_keyboardProcessor.start();

        collectData();

        keys = m_spiderManager.getSiteNames();

        while( keys.hasMoreElements() ){
            siteName = (String)keys.nextElement();
            site = m_spiderManager.getSite(siteName);
            siteStats = new SiteStats(site);

            reportData( site, siteStats );
        }
    }
//------------------------------------------------------------------------------
    private void collectData(){

        /*
         *  Starting the process manager
         */
        m_spiderManager.start();


        /*
         *  We wait until the m_spiderManager is done
         *  before we return from this method
         */
        try{
            m_spiderManager.join();
        }
        catch( InterruptedException intEx ){
            System.err.println( "Metis::collectData() : " + intEx );
        }

        /*
         *  Doing a bit of cleanup
         */
        System.gc();
        System.runFinalization();
    }
//------------------------------------------------------------------------------
    private void reportData( Site site, SiteStats stats ){
        MetisReport report = null;

        switch( MetisOpt.getReportType() ){
            case MetisGlobal.REPORT_TYPE_ASCII:
                report = (MetisReport)new MetisReportASCII( site, stats, site.getDomain() );
                break;
            case MetisGlobal.REPORT_TYPE_HTML:
                report = (MetisReport)new MetisReportHTML( site, stats );
                break;
            default:
                System.err.println("Unsupported report type. Using general ASCII report\n");
                report = (MetisReport)new MetisReportASCII( site, stats, site.getDomain() );
                break;
        }

        report.generateReport();
    }
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
    public static void main( String args[] ){
        Metis metis = null;
        MetisOpt options = new MetisOpt();

        /*
         *  Release 1.4.1:
         *      - Thanks for Richard Turner for pointing out the X option
         */
        Getopt opt = new Getopt("metis", args, "DRSRr:U:P:T:F:h:l:E:C:X:L:");
        int c;
        String argument = "";
        String cmlHost = null;
        String cmlEmailDomain = null;
        String cmlCustomLoginXMLFile = null;

        if( args.length < 1 )
            options.usage();

        while( (c = opt.getopt()) != -1 ){
            argument = "";
            switch( c ){
                case 'D':
                    options.setDebug(true);
                    break;
                case 'R':
                    // not implemented yet
                    break;
                case 'S':
                    // not implemented yet
                    break;
                case 'r':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a request method!");
                        options.usage();
                    }
                    try{
                        options.setRequestMethod( Integer.parseInt(argument) );
                    }
                    catch( NumberFormatException requestNumEx ){
                        System.err.println(requestNumEx);
                        System.exit(1);
                    }
                    break;
                case 'U':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a username!");
                        options.usage();
                    }
                    options.setRequestUserName(argument);
                    break;
                case 'P':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a password!");
                        options.usage();
                    }
                    options.setRequestPassword(argument);
                    break;
                case 'T':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a report type!");
                        options.usage();
                    }
                    try{
                        options.setReportType( Integer.parseInt(argument) );
                    }
                    catch( NumberFormatException reportNumEx ){
                        System.err.println(reportNumEx);
                        System.exit(1);
                    }
                    break;
                case 'h':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a site!");
                        options.usage();
                    }
                    cmlHost = argument;
                    //options.addSiteToList(argument);
                    break;
                case 'X':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a proxy host!");
                        options.usage();
                    }
                    options.setProxyHost(argument);
                    break;
                case 'C':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a proxy port!");
                        options.usage();
                    }
                    try{
                        Integer proxyPort = new Integer(argument);
                        options.setProxyPort( proxyPort.intValue() );
                    }
                    catch( NumberFormatException numEx ){
                        System.err.println("You need to specify a number for the proxy port ! : " + numEx );
                        options.usage();
                    }
                    break;
                case 'l':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a file name!");
                        options.usage();
                    }
                    options.setAndInitSiteFileList(argument);
                    break;
                case 'F':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a file name!");
                        options.usage();
                    }
                    options.setAndInitURLList(argument);
                    break;
                case 'E':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify a email domain!");
                        options.usage();
                    }
                    cmlEmailDomain = argument;
                    break;
                case 'L':
                    argument = opt.getOptarg();
                    if( argument == null ){
                        System.err.println("You need to specify the xml configuration file name");
                        options.usage();
                    }
                    cmlCustomLoginXMLFile = argument;
                    break;
                default:
                    System.err.println("Unknown options : " + (char)c);
                    options.usage();
                    break;
            }
        }

        if( cmlHost != null )       // host was specified on the command line
            options.addSiteToList(cmlHost, cmlEmailDomain, cmlCustomLoginXMLFile);

        //validate the options
        if( !options.validateOptions() ){
            options.usage();
        }

        try{
            metis = new Metis();
        }
        catch( MalformedURLException urlEx ){
            System.err.println("Metis::main() : " + urlEx );
            System.exit(0);
        }

        metis.run();

        System.exit(0);
    }

//------------------------------------------------------------------------------
    public void receiveEvent( ThreadEvent event ){
        Object result = null;
        Object source = null;
        int eventType = 0;
        Site site = null;
        SiteStats siteStats = null;

        result = event.getResult();
        source = event.getSource();
        eventType = event.getEventType();

        switch( eventType ){
            case ThreadEvent.EVENT_STOP:
                if( WebGlobal.DEBUG )
                    System.err.println( getName() + "::receiveEvent : GOT EVENT_STOP  from : " + source.getClass().getName() );

                setState( eventType );
                break;
            case ThreadEvent.EVENT_SITE_FINISH:
                if( WebGlobal.DEBUG )
                    System.err.println( getName() + "::receiveEvent : GOT EVENT_SITE_FINISH  from : " + source.getClass().getName() );

                setState(eventType);
                break;
            default:
                if( WebGlobal.DEBUG )
                    System.err.println( getName() + "::receiveEvent : got unreleated eventType : " + eventType + " from : " + source.getClass().getName() );

                break;
        }
    }
}
