/*
** Copyright (C) 2001,2002,2003 Sacha Faust <sacha@severus.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
 *  $Revision: 29 $
 *  $Modtime: 3/19/03 12:17a $
 *  Soft Version : 2.1
 */
package org.ideahamster.metis.report;

import org.ideahamster.metis.util.*;
import org.ideahamster.metis.stat.*;
import faust.sacha.web.data.*;
import faust.sacha.web.util.*;
import java.io.*;
import java.util.*;
import java.text.DecimalFormat;
/**
 * @author Sacha Faust
 * @version <PRE>2.1</PRE>
 */
public class MetisReportHTML extends MetisReport{

    private File m_reportFolder = null;
    private String BLANK_LINE = "&nbsp;";
    private String DATA_FOLDERNAME = "Data";


    public MetisReportHTML( Site site, SiteStats siteStats ){
        super(site, siteStats);
        m_site = site;
        m_siteStats = siteStats;

    }
//------------------------------------------------------------------------------
    private boolean createReportFolder( String name ){
        int id = 1;
        m_reportFolder = new File(name);
        String folderName = null;
        String tmpName = null;


        folderName = name;

        m_reportFolder = new File(name);

        while( m_reportFolder.exists() ){

            if( id == Integer.MAX_VALUE ){
                System.err.println("To many folders for this domain present!");
                return false;
            }

            tmpName = name + "_" + id;

            m_reportFolder = new File(tmpName);
            id++;
        }

        return m_reportFolder.mkdir();
    }
//------------------------------------------------------------------------------
    private String generateFramePage(){
        StringBuffer reportBuff = new StringBuffer();

        reportBuff.append( "<html>");
        reportBuff.append( "<head>");
        reportBuff.append( "<title>Metis report for " + m_site.getDomain() + "</title>");
        reportBuff.append( "</head>");

        reportBuff.append( "<frameset rows=\"56,602*\" cols=\"*\" frameborder=\"YES\" border=\"0\" framespacing=\"0\" bordercolor=\"#FFFFFF\">");
        reportBuff.append( "<frame name=\"topFrame\" scrolling=\"NO\" noresize src=\"top_frame.htm\" >");
        reportBuff.append( "<frameset cols=\"100,*\" frameborder=\"YES\" bordercolor=\"#CCCCCC\"> ");
        reportBuff.append( "<frame name=\"leftFrame\" scrolling=\"YES\" src=\"left_frame.htm\">");
        reportBuff.append( "<frame name=\"mainFrame\" src=\"center_frame.htm\">");
        reportBuff.append( "</frameset>");
        reportBuff.append( "</frameset>");
        reportBuff.append( "<noframes>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\" bordercolor=\"#FFFFFF\">");
        reportBuff.append( "</body></noframes>");
        reportBuff.append( "</html>" );

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
    private String generateLeftFrame(){
        StringBuffer reportBuff = new StringBuffer();

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");
        reportBuff.append( "<div align=\"center\">");
        reportBuff.append( "<p><a href=\"center_frame.htm\" target=\"mainFrame\">General</a></p>");
        reportBuff.append( "<p><a href=\"" + DATA_FOLDERNAME + File.separator + "file_by_type.htm\" target=\"mainFrame\">Files by Type</a> </p>");
        reportBuff.append( "<p><a href=\"" + DATA_FOLDERNAME + File.separator + "folder.htm\" target=\"mainFrame\">Folders</a> </p>");
        reportBuff.append( "<p><a href=\"" + DATA_FOLDERNAME + File.separator + "file_info_by_folder.htm\" target=\"mainFrame\">File Info by Folders</a> </p>");
        reportBuff.append( "<p><a href=\"" + DATA_FOLDERNAME + File.separator + "comments_by_file.htm\" target=\"mainFrame\">Comments by Files</a> </p>");
        reportBuff.append( "<p><a href=\"" + DATA_FOLDERNAME + File.separator + "emails_by_domain.htm\" target=\"mainFrame\">Emails by Domain</a> </p>");
        reportBuff.append( "<p><a href=\"" + DATA_FOLDERNAME + File.separator + "site_scripts.htm\" target=\"mainFrame\">Site scripts</a> </p>");
        reportBuff.append( "<p><a href=\"" + DATA_FOLDERNAME + File.separator + "outside_url_by_domain.htm\" target=\"mainFrame\">Outside urls by domain</a> </p>");
        reportBuff.append( "<p><a href=\"" + DATA_FOLDERNAME + File.separator + "site_users.htm\" target=\"mainFrame\">Users found</a> </p>");
        reportBuff.append( "<p><a href=\"" + DATA_FOLDERNAME + File.separator + "keywords_found.htm\" target=\"mainFrame\">Keywords found</a> </p>");
        reportBuff.append( "<p><a href=\"" + DATA_FOLDERNAME + File.separator + "site_map.htm\" target=\"mainFrame\">Site Map</a> </p>");
        reportBuff.append( "</div>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
    private String generateTopFrame(){
        StringBuffer reportBuff = new StringBuffer();

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");
        reportBuff.append( "<div align=\"center\"><font size=\"+2\">" + m_site.getDomain() + " Metis report.</font> </div>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
    private String getHTMLLinkCode( String link ){
        return "<a href=\"" + link + "\">" + link + "</a>";
    }
//------------------------------------------------------------------------------
    private String getHTMLLinkCode( String link, String text ){
        return "<a href=\"" + link + "\">" + text + "</a>";
    }
//------------------------------------------------------------------------------
    private String getHTMLEmailLinkCode( String link ){
        return "<a href=\"mailto:" + link + "\">" + link + "</a>";
    }
//------------------------------------------------------------------------------
    private String getHTMLEmailLinkCode( EmailURL email ){
        return getHTMLEmailLinkCode(email.getEmailURL());
    }
//------------------------------------------------------------------------------
    protected String generateHeaderReport() {
        StringBuffer reportBuff = new StringBuffer();
        double totalSiteSize = 0;
        String totalSiteSizeStr = null;
        int dotPos = 0;
        DecimalFormat format = new DecimalFormat("0.00");

        /*
         *  Note:
         *      this <html> and the following <body> tags will be ended
         *      in generateFootNote()
         */
        reportBuff.append("<html>" + "\n"); //
        reportBuff.append("<body bgcolor=\"#FFFFFF\">" + "\n");
        reportBuff.append("<p align=\"left\">Site : " + m_site.getDomain() + "</p>" + "\n");
        reportBuff.append("<p align=\"left\">Server type : " + m_siteStats.getSiteServerType() + "</p>" + "\n");
        reportBuff.append("<p align=\"left\">Date : " + new Date().toString());
        reportBuff.append("<p align=\"left\">Total Number of Files : " + m_siteStats.getNumberOfFiles());
        reportBuff.append("<p align=\"left\">Total Number of Folders : " + m_siteStats.getNumberOfFolders());
        reportBuff.append("<p align=\"left\">Total Number of E-mails : " + m_siteStats.getNumberOfEmails());

        totalSiteSize = (double)m_siteStats.getTotalObjectSize();
        totalSiteSize = totalSiteSize / (double)1048576;
        if( totalSiteSize < 1 )
            reportBuff.append("<p align=\"left\">Site size estimate : " + format.format(totalSiteSize) + " Bytes");
        else
            reportBuff.append("<p align=\"left\">Site size estimate : " + format.format(totalSiteSize) + " MB");

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
     protected String generateFootNote(){
         StringBuffer reportBuff = new StringBuffer();

         // spacing
         reportBuff.append( "<p>" + BLANK_LINE + "</p>" + "\n");
         reportBuff.append( "<p>" + BLANK_LINE + "</p>" + "\n");
         reportBuff.append( "<p>" + BLANK_LINE + "</p>" + "\n");

         reportBuff.append( "<hr align=\"center\" noshade>");
         reportBuff.append( "<p><font size=\"2\">This report was generated by " +
                            MetisGlobal.METIS_USER_AGENT + " written by " + MetisGlobal.METIS_AUTHOR + " : </font></p>"  );
         reportBuff.append( getHTMLEmailLinkCode(MetisGlobal.METIS_AUTHOR_EMAIL));

         reportBuff.append( "</body>");
         reportBuff.append( "</html>");

         return reportBuff.toString();
     }
//------------------------------------------------------------------------------
    protected String generateEmailByDomainReport() {
        StringBuffer reportBuff = new StringBuffer();
        EmailByDomainStat emailStat = m_siteStats.getEmailByDomainStats();
        Enumeration domainsKeys = emailStat.getDataKeys();
        String domain = null;
        ElementStat elementData = null;
        ArrayList elementArray = null;
        EmailURL email = null;

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");
        reportBuff.append( "<table width=\"75%\" border=\"1\" bordercolor=\"#FFFFFF\">");

        while( domainsKeys.hasMoreElements() ){
            domain = (String)domainsKeys.nextElement();
            elementData = emailStat.getElement(domain);
            elementArray = elementData.getMembers();

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>Domain</b></td>");
            reportBuff.append( "<td>" + domain + "</td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>" + "\n");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>" + "\n");
            reportBuff.append( "</tr>");

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>E-mails</b></td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "<td><b>Subject</b></td>");
            reportBuff.append( "<td><b>Arguments</b></td>");
            reportBuff.append( "</tr>");

            for( int i = 0; i < elementArray.size(); i++ ){
                email = (EmailURL)elementArray.get(i);

                reportBuff.append( "<tr>");
                reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                reportBuff.append( "<td>" + getHTMLEmailLinkCode(email) + "</td>");
                reportBuff.append( "<td>" + email.getSubject() + "</td>");
                reportBuff.append( "<td>" + email.getArgumentData() + "</td>");
                reportBuff.append( "</tr>");
            }
        }
        reportBuff.append( "</table>" );
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
    protected String generateFileInfoReportByFolder() {
        StringBuffer reportBuff = new StringBuffer();
        FolderStat folderStat = m_siteStats.getFolderStats();
        Enumeration folderKeys = folderStat.getDataKeys();
        String key = null;
        ElementStat elementData = null;
        ArrayList fileList = null;
        FileURL file = null;
        ArrayList queryList = null;
        ScriptStateInfo queryData = null;

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");
        reportBuff.append( "<table width=\"75%\" border=\"1\" bordercolor=\"#FFFFFF\">");

        while( folderKeys.hasMoreElements() ){
            key = (String)folderKeys.nextElement();
            elementData = folderStat.getElement(key);
            fileList = elementData.getMembers();

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>Folder Name</b></td>");
            reportBuff.append( "<td>" + key + "</td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "</tr>");

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>Files</b></td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "</tr>");

            reportBuff.append( "<tr>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "<td><b>File Name</b></td>");
            reportBuff.append( "<td><b>Size</b></td>");
            reportBuff.append( "<td><b>Last Modified</b></td>");
            reportBuff.append( "</tr>");

            for( int i = 0; i < fileList.size(); i++ ){
                file = (FileURL)fileList.get(i);
                reportBuff.append( "<tr>");
                reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                reportBuff.append( "<td>" + getHTMLLinkCode(file.toString(), file.getFileName()) + "</td>");
                if( file.isScript() ){
                    /*
                     *  finishing the table entry
                     */
                    reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                    reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                    reportBuff.append( "</tr>");

                    reportBuff.append( "<tr>");
                    reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                    reportBuff.append( "<td><b>Queries</b></td>" );
                    reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                    reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                    queryList = file.getQueryDataList();

                    for( int queryIndex = 0; queryIndex < queryList.size(); queryIndex++ ){
                        queryData = (ScriptStateInfo)queryList.get(queryIndex);

                        reportBuff.append( "<tr>");
                        reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                        reportBuff.append( "<td>" + getHTMLLinkCode( queryData.getURL(), queryData.getQuery() ) + "</td>");
                        reportBuff.append( "<td>" + queryData.getContentLengthInfoAsString() + "</td>");
                        reportBuff.append( "<td>" + queryData.getLastModifiedInfo() + "<td>");
                        reportBuff.append( "</tr>");
                    }
                }
                else{
                    reportBuff.append( "<td>" + file.getContentLengthInfoAsString() + "</td>");
                    reportBuff.append( "<td>" + file.getLastModifiedInfo() + "<td>");
                    reportBuff.append( "</tr>");
                }
            }
        }
        reportBuff.append( "</table>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
    protected String generateFolderReport() {
        StringBuffer reportBuff = new StringBuffer();
        FolderStat folderStat = m_siteStats.getFolderStats();
        Enumeration folderKeys = folderStat.getDataKeys();
        String key = null;
        ElementStat elementData = null;
        DecimalFormat format = new DecimalFormat("0.00");

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");

        reportBuff.append( "<table width=\"75%\" border=\"1\" bordercolor=\"#FFFFFF\">");

        while( folderKeys.hasMoreElements() ){
            key = (String)folderKeys.nextElement();
            elementData = folderStat.getElement(key);

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>Folder name</b></td>");
            reportBuff.append( "<td>" + getHTMLLinkCode(m_site.getInfo().getBaseURL() + key, key) + "</td>");
            reportBuff.append( "</tr>");

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>Number of files</b></td>");
            reportBuff.append( "<td>" + elementData.getNumberOfElements() + "</td>");
            reportBuff.append( "</tr>");

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>% on Total</b></td>");
            reportBuff.append( "<td>" + format.format(elementData.getPercentage()) + "</td>");
            reportBuff.append( "</tr>");

            //spacing
            reportBuff.append( "<tr>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "</tr>");
        }
        reportBuff.append( "</table>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
    protected String generateSiteMapReport() {
        StringBuffer reportBuff = new StringBuffer();
        Enumeration folderKeys = m_site.getFoldersNames();
        String folderName = null;
        ArrayList fileArray = null;
        FileURL file = null;
        ArrayList queryList = null;
        ScriptStateInfo queryData = null;

        reportBuff.append( m_site.getSiteURL());

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");

        reportBuff.append( getHTMLLinkCode(m_site.getInfo().getBaseURL()));
        reportBuff.append( "<blockquote>");

        while( folderKeys.hasMoreElements() ){
            folderName = (String)folderKeys.nextElement();
            fileArray = m_site.getFolderElements(folderName);

            //folder
            reportBuff.append( getHTMLLinkCode(m_site.getInfo().getBaseURL() + folderName));
            reportBuff.append( "<blockquote>");

            //files
            for( int i = 0; i < fileArray.size(); i++ ){
                file = (FileURL)fileArray.get(i);
                //queryList = file.getQ
                queryList = file.getQueryDataList();

                reportBuff.append( (i == 0 ? "" : "<br>") + getHTMLLinkCode(file.toString()));
                //reportBuff.append( "<br>" + getHTMLLinkCode(file.toString()));

                for( int queryIndex = 0; queryIndex < queryList.size(); queryIndex++ ){

                    queryData = (ScriptStateInfo)queryList.get(queryIndex);

                    if( queryIndex == 0 )
                        reportBuff.append( "<blockquote>");

                    reportBuff.append( (queryIndex == 0 ? "" : "<br>") + getHTMLLinkCode(queryData.getURL()));
                }

                if( file.isScript() ){
                    //query
                    reportBuff.append( "</blockquote>");
                }
            }
            //folder
            reportBuff.append( "</blockquote>");
        }

        reportBuff.append( "</blockquote>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
    protected String generateFileByTypeReport() {
        StringBuffer reportBuff = new StringBuffer();
        FileExtensionStat fileExStats = m_siteStats.getFileExtensionStats();
        Enumeration keys = fileExStats.getDataKeys();
        String fileExKey = null;
        ElementStat elementData = null;
        ArrayList fileList = null;
        FileURL file = null;
        DecimalFormat format = new DecimalFormat("0.00");

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");
        reportBuff.append( "<table width=\"75%\" border=\"1\" bordercolor=\"#FFFFFF\">" );

        while( keys.hasMoreElements() ){
            fileExKey = (String)keys.nextElement();
            elementData = (ElementStat)fileExStats.getElement( fileExKey );
            fileList = elementData.getMembers();

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>Name</b></td>");
            reportBuff.append( "<td>" + fileExKey + "</td>");
            reportBuff.append( "</tr>");

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>Number of files</b></td>");
            reportBuff.append( "<td>" + elementData.getNumberOfElements() + "</td>");
            reportBuff.append( "</tr>");

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>% on Total</b></td>");
            reportBuff.append( "<td>" + format.format(elementData.getPercentage()) + "</td>");
            reportBuff.append( "</tr>");

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>File List</b></td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "</tr>");

            for( int i = 0; i < fileList.size(); i++ ){
                file = (FileURL)fileList.get(i);

                reportBuff.append( "<tr>");
                reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                reportBuff.append( "<td>" + getHTMLLinkCode(file.toString(), file.getPath() ) + "</td>");
                reportBuff.append( "</tr>");
            }
        }
        reportBuff.append( "</table>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
     protected String generateCommentsByFileReport() {
        StringBuffer reportBuff = new StringBuffer();
        Enumeration folderKeys = m_site.getFoldersNames();
        String key = null;
        ArrayList fileList = null;
        FileURL file = null;
        ArrayList commentList = null;
        ArrayList queryList = null;
        String query = null;
        ScriptStateInfo queryData = null;
        String comment = null;


        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");
        reportBuff.append( "<table width=\"75%\" border=\"1\" bordercolor=\"#FFFFFF\">");

        while( folderKeys.hasMoreElements() ){
            key = (String)folderKeys.nextElement();
            fileList = m_site.getFolderElements(key);
            for( int fileIndex = 0; fileIndex < fileList.size(); fileIndex++ ){
                file = (FileURL)fileList.get(fileIndex);

                if( !file.isScript() ){
                    commentList = file.getComments();
                    if( commentList.size() == 0 )
                        continue;
                }

                reportBuff.append( "<tr>");
                reportBuff.append( "<td><b>File name</b></td>");
                reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                reportBuff.append( "</tr>");

                reportBuff.append( "<tr>");
                reportBuff.append( "<td>" + getHTMLLinkCode(file.toString(), file.getFolder() + file.getFileName()) + "</td>");

                if( file.isScript() )
                    reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                else
                    reportBuff.append( "<td><b>Comments</b></td>");

                reportBuff.append( "</tr>");

                if( file.isScript() ){
                    queryList = file.getQueryDataList();

                    for(int queryIndex = 0; queryIndex < queryList.size(); queryIndex++ ){
                        queryData = (ScriptStateInfo)queryList.get(queryIndex);


                        if( queryData == null ){
                            System.err.println("MetisReportHTML::generateCommentByFileReport() : queryData is null");
                            continue;
                        }

                        if( (query = queryData.getQuery()) == "" ){
                            System.err.println("MetisReportHTML::generateCommentByFileReport() : query is empty on : " + queryData.getURL() );
                            continue;
                        }

                        reportBuff.append( "<tr>");
                        reportBuff.append( "<td><b>Query</b></td>");
                        reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                        reportBuff.append( "</tr>");

                        reportBuff.append( "<tr>");
                        reportBuff.append( "<td>" + getHTMLLinkCode(queryData.getURL(), file.getFolder() + file.getFileName() + query) + "</td>");
                        reportBuff.append( "<td><b>Comments</b></td>");
                        reportBuff.append( "</tr>");

                        commentList = queryData.getComments();
                        for( int commentIndex = 0; commentIndex < commentList.size(); commentIndex++ ){
                            comment = (String)commentList.get(commentIndex);

                            reportBuff.append( "<tr>");
                            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                            // adding <xmp></xmp> to make sure we don't include html that is in the comment
                            // Tks Prizm for pointing out this tag and make me dump <pre>
                            reportBuff.append( "<td><xmp>" + comment + "</xmp></td>");
                            reportBuff.append( "</tr>");
                        }
                    }
                }
                for( int commentIndex = 0; commentIndex < commentList.size(); commentIndex++ ){
                    comment = (String)commentList.get(commentIndex);

                    reportBuff.append( "<tr>");
                    reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                    // adding <xmp></xmp> to make sure we don't include html that is in the comment
                    // Tks Prizm for pointing out this tag and make me dump <pre>
                    reportBuff.append( "<td><xmp>" + comment + "</xmp></td>");
                    reportBuff.append( "</tr>");
                }
            }
            reportBuff.append( "<tr>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "</tr>");
        }

        reportBuff.append( "</table>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
     }
//------------------------------------------------------------------------------
    protected String generateOutsideURLByDomainReport(){
        StringBuffer reportBuff = new StringBuffer();
        OutsideURLByDomainStat urlStat = m_siteStats.getOutsideURLByDomainStats();
        Enumeration domainKeys = urlStat.getDataKeys();
        String key = null;
        ElementStat elementData = null;
        ArrayList urlList = null;
        URLInfo url = null;

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");
        reportBuff.append( "<table width=\"%75\" border=\"1\" bordercolor=\"#FFFFFF\">");

        while( domainKeys.hasMoreElements() ){
            key = (String)domainKeys.nextElement();
            elementData = urlStat.getElement(key);
            urlList = elementData.getMembers();

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>Domain</b></td>");
            reportBuff.append( "<td>" + key + "</td>");
            reportBuff.append( "</tr>");

            for( int i = 0; i < urlList.size(); i++ ){
                url = (URLInfo)urlList.get(i);
                reportBuff.append( "<tr>");
                reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                reportBuff.append( "<td>" + getHTMLLinkCode(url.getURL()) + "<td>");
                reportBuff.append( "</tr>");
            }
        }
        reportBuff.append( "</table>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
    protected String generateScriptReport(){
        StringBuffer reportBuff = new StringBuffer();
        SiteScripts siteScripts = m_siteStats.getSiteScripts();
        ArrayList scriptList = siteScripts.getScripts();
        FileURL script = null;
        Enumeration varNames = null;
        String varKey = null;
        ArrayList dataList = null;
        String data = null;

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");
        reportBuff.append( "<table width=\"%75\" border=\"1\" bordercolor=\"#FFFFFF\">");

        for( int scriptIndex = 0; scriptIndex < scriptList.size(); scriptIndex++ ){
            script = (FileURL)scriptList.get(scriptIndex);
            varNames = script.getArgumentVarNames();

            //show the script in folder/script format
            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>Script file</b></td>");
            reportBuff.append( "<td>" + getHTMLLinkCode(script.toString(), script.getFolder() + script.getFileName()) + "</td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "</tr>");

            while( varNames.hasMoreElements() ){
                varKey = (String) varNames.nextElement();
                dataList = script.getArgumentVarDataList(varKey);

                reportBuff.append( "<tr>");
                reportBuff.append( "<td><b>Variable name</b></td>");
                reportBuff.append( "<td>" + varKey + "</td>");
                reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                reportBuff.append( "</tr>");

                reportBuff.append( "<tr>");
                reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                reportBuff.append( "<td><b>Instances</b></td>");
                reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                reportBuff.append( "</tr>");

                for( int dataIndex = 0; dataIndex < dataList.size(); dataIndex++ ){
                    data = (String)dataList.get(dataIndex);
                    if( data == "" )
                        continue;

                    reportBuff.append( "<tr>");
                    reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                    reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                    reportBuff.append( "<td>" + data + "</td>");
                    reportBuff.append( "</tr>");
                }
            }
        }
        reportBuff.append( "</table>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
    }
//------------------------------------------------------------------------------
     protected String generateSiteUserList() {
        StringBuffer reportBuff = new StringBuffer();
        Enumeration keys = null;
        String method = null;
        Vector tmpList = null;
        SiteUsers userList = null;
        String tmpUser = null;

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");
        reportBuff.append( "<table width=\"%75\" border=\"1\" bordercolor=\"#FFFFFF\">");

        userList = m_siteStats.getSiteUsers();
        keys = userList.getUserListKeys();

        while( keys.hasMoreElements() ){
            method = (String)keys.nextElement();
            tmpList = userList.getUserList(method);

            if( method.equals("emails") )
                method += " (" + m_site.getEmailDomain() + ")";

            reportBuff.append( "<tr>");
            reportBuff.append( "<td><b>Found via : " + method + "</b></td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "</tr>");

            for( int i = 0; i < tmpList.size(); i++ ){
                tmpUser = (String)tmpList.get(i);
                reportBuff.append( "</tr>");
                reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                reportBuff.append( "<td>" + tmpUser + "</td>");
                reportBuff.append( "</tr>");
            }

            reportBuff.append( "</tr>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "<td>" + BLANK_LINE + "</td>");
            reportBuff.append( "</tr>");
        }
        reportBuff.append( "</table>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
     }
//------------------------------------------------------------------------------
     protected String generateSiteKeywordList() {
        StringBuffer reportBuff = new StringBuffer();
        String key = null;
        Enumeration keywordKeys = m_site.getKeywordKeys();
        List urlList = null;
        String url = null;

        reportBuff.append( "<html>");
        reportBuff.append( "<body bgcolor=\"#FFFFFF\">");
        reportBuff.append( "<table width=\"%75\" border=\"1\" bordercolor=\"#FFFFFF\">");

        while( keywordKeys.hasMoreElements() ){
            key = (String)keywordKeys.nextElement();
            urlList = (List)m_site.getKeywordURL(key);

            reportBuff.append( "<tr>" );
            reportBuff.append( "<td><b>Keyword</b></trd>" );
            reportBuff.append( "<td>" + key + "</td");
            reportBuff.append( "</tr>" );

            for( int i = 0; i < urlList.size(); i++ ){
                url = (String)urlList.get(i);
                reportBuff.append( "<tr>");
                reportBuff.append( "<td>" + BLANK_LINE + "</td>");
                reportBuff.append( "<td>" + getHTMLLinkCode(url) + "<td>");
                reportBuff.append( "</tr>");
            }
        }

        reportBuff.append( "</table>");
        reportBuff.append( "</body>");
        reportBuff.append( "</html>");

        return reportBuff.toString();
     }
//------------------------------------------------------------------------------
     /*
      * Overload parent to handle proper files creation
      */
     public void generateReport(){
         File dataFolder = null;
         RandomAccessFile topFrame = null;
         RandomAccessFile leftFrame = null;
         RandomAccessFile centerFrame = null;
         RandomAccessFile emailsByDomainReport = null;
         RandomAccessFile fileByTypeReport = null;
         RandomAccessFile fileInfoByFolderReport = null;
         RandomAccessFile commentInfoByFileReport = null;
         RandomAccessFile folderReport = null;
         RandomAccessFile outsideUrlByDomainReport = null;
         RandomAccessFile siteMapReport = null;
         RandomAccessFile siteScriptsReport = null;
         RandomAccessFile siteUsersReport = null;
         RandomAccessFile keywordReport = null;

         if( !createReportFolder(m_site.getDomain()) ){
            System.err.println("Unable to create report folder!");
            System.exit(1);
         }

         dataFolder = new File(m_reportFolder, DATA_FOLDERNAME);
         if( !dataFolder.mkdir() ){
            System.err.println("Unable to create report data folder!");
            System.exit(1);
         }

         try{
             super.setFile( new RandomAccessFile( new File(m_reportFolder, m_site.getDomain() + ".htm"), "rw") );
             topFrame = new RandomAccessFile( new File(m_reportFolder, "top_frame.htm"), "rw" );
             leftFrame = new RandomAccessFile( new File(m_reportFolder, "left_frame.htm"), "rw" );
             centerFrame = new RandomAccessFile( new File(m_reportFolder, "center_frame.htm"), "rw" );

             emailsByDomainReport = new RandomAccessFile( new File(dataFolder, "emails_by_domain.htm"), "rw" );
             fileByTypeReport = new RandomAccessFile( new File(dataFolder, "file_by_type.htm"), "rw" );
             fileInfoByFolderReport = new RandomAccessFile( new File(dataFolder, "file_info_by_folder.htm"), "rw" );
             commentInfoByFileReport = new RandomAccessFile( new File(dataFolder, "comments_by_file.htm"), "rw" );
             folderReport = new RandomAccessFile( new File(dataFolder, "folder.htm"), "rw" );
             outsideUrlByDomainReport = new RandomAccessFile( new File(dataFolder, "outside_url_by_domain.htm"), "rw" );
             siteMapReport = new RandomAccessFile( new File(dataFolder, "site_map.htm"), "rw" );
             siteScriptsReport = new RandomAccessFile( new File(dataFolder, "site_scripts.htm"), "rw" );
             siteUsersReport = new RandomAccessFile( new File(dataFolder, "site_users.htm"), "rw" );
             keywordReport = new RandomAccessFile( new File(dataFolder, "keywords_found.htm"), "rw" );
         }
         catch( FileNotFoundException notFoundEx ){
            System.err.println(notFoundEx);
            System.exit(1);
         }

         try{
             m_reportFile.writeBytes( generateFramePage() );
             topFrame.writeBytes( generateTopFrame() );
             leftFrame.writeBytes( generateLeftFrame() );
             centerFrame.writeBytes( generateHeaderReport() );
             centerFrame.writeBytes( generateFootNote() );

             emailsByDomainReport.writeBytes( generateEmailByDomainReport() );
             fileByTypeReport.writeBytes( generateFileByTypeReport() );
             fileInfoByFolderReport.writeBytes( generateFileInfoReportByFolder() );
             commentInfoByFileReport.writeBytes( generateCommentsByFileReport() );
             folderReport.writeBytes( generateFolderReport() );
             outsideUrlByDomainReport.writeBytes( generateOutsideURLByDomainReport() );
             siteMapReport.writeBytes( generateSiteMapReport() );
             siteScriptsReport.writeBytes( generateScriptReport() );
             siteUsersReport.writeBytes( generateSiteUserList() );
             keywordReport.writeBytes( generateSiteKeywordList() );
         }
         catch( IOException ioExWrite ){
             System.err.println(ioExWrite);
         }
         finally{
             try{
                 m_reportFile.close();
                 topFrame.close();
                 leftFrame.close();
                 centerFrame.close();
                 centerFrame.close();

                 emailsByDomainReport.close();
                 fileByTypeReport.close();
                 fileInfoByFolderReport.close();
                 commentInfoByFileReport.close();
                 folderReport.close();
                 outsideUrlByDomainReport.close();
                 siteMapReport.close();
                 siteScriptsReport.close();
                 siteUsersReport.close();
                 keywordReport.close();
             }
             catch( IOException ioExClose ){
             }
         }
     }
//------------------------------------------------------------------------------
}
